#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Utilities to process and convert svg images to png using palette colors.
"""

# Standard library imports

import logging
import os
import re
import shutil
import tempfile

# Third party imports
from qtpy.QtCore import QSize
from qtpy.QtGui import QIcon
from qtpy.QtWidgets import QApplication

# Local imports
from qrainbowstyle import (IMAGES_PATH, STYLES_SCSS_FILEPATH, QRC_FILEPATH, RC_PATH,
                           SVG_PATH, BUTTONS_NT_PATH, BUTTONS_DARWIN_PATH)
from qrainbowstyle.palette import BasePalette

IMAGE_BLACKLIST = ['base_palette']

TEMPLATE_QRC_HEADER = '''
<RCC warning="File created programmatically. All changes made in this file will be lost!">
  <qresource prefix="{resource_prefix}">
'''

TEMPLATE_QRC_FILE = '    <file>rc/{fname}</file>'

TEMPLATE_QRC_FOOTER = '''
  </qresource>
  <qresource prefix="{style_prefix}">
      <file>style.qss</file>
  </qresource>
</RCC>
'''

_logger = logging.getLogger(__name__)


def _create_nt_buttons(base_svg_path=BUTTONS_NT_PATH, rc_path=RC_PATH, palette=BasePalette):
    """Create png images from svg files for windows style buttons"""

    sizes = [{"ext": '.png', "width": 45, "height": 30}, {"ext": '@2x.png', "width": 90, "height": 60}]

    temp_dir = tempfile.mkdtemp()
    svg_fnames = [f for f in os.listdir(base_svg_path) if f.endswith('.svg')]

    background = palette.TITLE_BAR_BACKGROUND_COLOR
    background_hover = palette.TITLE_BAR_BUTTONS_HOVER_COLOR
    disabled = palette.TITLE_BAR_BUTTONS_DISABLED_COLOR
    text = palette.TITLE_BAR_TEXT_COLOR

    for svg in svg_fnames:
        svg_path = os.path.join(base_svg_path, svg)
        with open(svg_path, 'r') as fh:
            data = fh.read()

            new_data = data
            new_data = new_data.replace("TITLE_BAR_BUTTONS_DISABLED_COLOR", disabled)
            new_data = new_data.replace("COLOR_BACKGROUND_DARK", background)
            new_data = new_data.replace("COLOR_BACKGROUND_NORMAL", background_hover)
            new_data = new_data.replace("COLOR_FOREGROUND_LIGHT", text)

        temp_svg_path = os.path.join(temp_dir, svg)
        with open(temp_svg_path, 'w') as fh:
            fh.write(new_data)

        for size in sizes:
            png_fname = svg.replace('.svg', size['ext'])
            png_path = os.path.join(rc_path, png_fname)
            convert_svg_to_png(temp_svg_path, png_path, size['width'], size['height'])


def _create_darwin_buttons(base_svg_path=BUTTONS_DARWIN_PATH, rc_path=RC_PATH):
    """Create png images from svg files for darwin style buttons"""

    temp_dir = tempfile.mkdtemp()
    svg_fnames = [f for f in os.listdir(base_svg_path) if f.endswith('.svg')]

    sizes = [{"ext": '.png', "width": 32, "height": 32}, {"ext": '@2x.png', "width": 64, "height": 64}]

    for svg in svg_fnames:
        svg_path = os.path.join(base_svg_path, svg)
        temp_svg_path = os.path.join(temp_dir, svg)
        shutil.copy2(svg_path, temp_svg_path)

        for size in sizes:
            png_fname = svg.replace('.svg', size['ext'])
            png_path = os.path.join(rc_path, png_fname)
            convert_svg_to_png(temp_svg_path, png_path, size['height'], size['width'])


def create_titlebar_images(buttons_svg_path=SVG_PATH, rc_path=RC_PATH, palette=BasePalette):
    """Create resources `rc` png image files from titlebar buttons svg files and palette
    """
    _ = QApplication([])
    _create_nt_buttons(rc_path=rc_path, palette=palette)
    _create_darwin_buttons(rc_path=rc_path)


def _get_file_color_map(fname, palette):
    """
    Return map of files (i.e states) to color from given palette.
    """
    color_disabled = palette.COLOR_BACKGROUND_4
    color_focus = palette.COLOR_ACCENT_5
    color_pressed = palette.COLOR_ACCENT_2
    color_normal = palette.COLOR_TEXT_1

    name, ext = fname.split('.')
    file_colors = {
        fname: color_normal,
        name + '_disabled.' + ext: color_disabled,
        name + '_focus.' + ext: color_focus,
        name + '_pressed.' + ext: color_pressed,
    }

    return file_colors


def _create_colored_svg(svg_path, temp_svg_path, color):
    """
    Replace base svg with fill color.
    """
    with open(svg_path, 'r') as fh:
        data = fh.read()

    base_color = '#ff0000'  # Hardcoded in base svg files
    new_data = data.replace(base_color, color)

    with open(temp_svg_path, 'w') as fh:
        fh.write(new_data)


def convert_svg_to_png(svg_path, png_path, height, width):
    """
    Convert svg files to png files using Qt.
    """
    size = QSize(height, width)
    icon = QIcon(svg_path)
    pixmap = icon.pixmap(size)
    img = pixmap.toImage()
    img.save(png_path)


def create_palette_image(base_svg_path=SVG_PATH, path=IMAGES_PATH,
                         palette=BasePalette):
    """
    Create palette image svg and png image on specified path.
    """
    # Needed to use QPixmap
    _ = QApplication([])

    base_palette_svg_path = os.path.join(base_svg_path, 'base_palette.svg')
    palette_svg_path = os.path.join(path, 'palette.svg')
    palette_png_path = os.path.join(path, 'palette.png')

    _logger.info("Creating palette image ...")
    _logger.info("Base SVG: %s", base_palette_svg_path)
    _logger.info("To SVG: %s", palette_svg_path)
    _logger.info("To PNG: %s", palette_png_path)

    with open(base_palette_svg_path, 'r') as fh:
        data = fh.read()

    color_palette = palette.color_palette()

    for color_name, color_value in color_palette.items():
        data = data.replace('{{ ' + color_name + ' }}', color_value.lower())

    with open(palette_svg_path, 'w+') as fh:
        fh.write(data)

    convert_svg_to_png(palette_svg_path, palette_png_path, 4000, 4000)

    return palette_svg_path, palette_png_path


def create_images(base_svg_path=SVG_PATH, rc_path=RC_PATH,
                  palette=BasePalette):
    """Create resources `rc` png image files from base svg files and palette.

    Search all SVG files in `base_svg_path` excluding IMAGE_BLACKLIST,
    change its colors using `palette` creating temporary SVG files, for each
    state generating PNG images for each size `heights`.

    Args:
        base_svg_path (str, optional): Input svgs directory path. Defaults to SVG_PATH.
        rc_path (str, optional): Output pngs directory path. Defaults to RC_PATH.
        palette (BasePalette, optional): Palette . Defaults to BasePalette.
    """

    # Needed to use QPixmap
    _ = QApplication([])

    temp_dir = tempfile.mkdtemp()
    svg_fnames = [f for f in os.listdir(base_svg_path) if f.endswith('.svg')]
    base_height = 32

    # See: https://doc.qt.io/qt-5/scalability.html
    heights = {
        32: '.png',
        64: '@2x.png',
    }

    _logger.info("Creating images ...")
    _logger.info("SVG folder: %s", base_svg_path)
    _logger.info("TMP folder: %s", temp_dir)
    _logger.info("PNG folder: %s", rc_path)

    num_svg = len(svg_fnames)
    num_png = 0
    num_ignored = 0

    # Get rc links from scss to check matches
    rc_list = get_rc_links_from_scss()
    num_rc_list = len(rc_list)

    for height, ext in heights.items():
        width = height

        _logger.debug(" Size HxW (px): {} X {}".format(height, width))

        for svg_fname in svg_fnames:
            svg_name = svg_fname.split('.')[0]

            # Skip blacklist
            if svg_name not in IMAGE_BLACKLIST:
                svg_path = os.path.join(base_svg_path, svg_fname)
                color_files = _get_file_color_map(svg_fname, palette=palette)

                _logger.debug("  Working on: %s"
                              % os.path.basename(svg_fname))

                # Replace colors and create all file for different states
                for color_svg_name, color in color_files.items():
                    temp_svg_path = os.path.join(temp_dir, color_svg_name)
                    _create_colored_svg(svg_path, temp_svg_path, color)

                    png_fname = color_svg_name.replace('.svg', ext)
                    png_path = os.path.join(rc_path, png_fname)
                    convert_svg_to_png(temp_svg_path, png_path, height, width)
                    num_png += 1
                    _logger.debug("   Creating: %s"
                                  % os.path.basename(png_fname))

                    # Check if the rc_name is in the rc_list from scss
                    # only for the base size
                    if height == base_height:
                        rc_base = os.path.basename(rc_path)
                        png_base = os.path.basename(png_fname)
                        rc_name = '/' + os.path.join(rc_base, png_base)
                        try:
                            rc_list.remove(rc_name)
                        except ValueError:
                            pass
            else:
                num_ignored += 1
                _logger.debug("  Ignored blacklist: %s"
                              % os.path.basename(svg_fname))

    _logger.info("# SVG files: %s", num_svg)
    _logger.info("# SVG ignored: %s", num_ignored)
    _logger.info("# PNG files: %s", num_png)
    _logger.info("# RC links: %s", num_rc_list)
    _logger.info("# RC links not in RC: %s", len(rc_list))
    _logger.info("RC links not in RC: %s", rc_list)


def generate_qrc_file(resource_prefix='qss_icons', style_prefix='qrainbowstyle',
                      rc_path=RC_PATH, qrc_path=QRC_FILEPATH):
    """
    Generate the QRC file programmaticaly.

    Search all RC folder for PNG images and create a QRC file.

    Args:
        resource_prefix (str, optional): Prefix used in resources.
            Defaults to 'qss_icons'.
        style_prefix (str, optional): Prefix used to this style.
            Defaults to 'qrainbowstyle'.
        rc_path (str, optional): Source path
            Defaults to 'RC_PATH'
        qrc_path (str, optional): Output path
            Defaults to 'QRC_FILEPATH'
    """

    files = []

    _logger.info("Generating QRC file ...")
    _logger.info("Resource prefix: %s", resource_prefix)
    _logger.info("Style prefix: %s", style_prefix)

    _logger.info("Searching in: %s", rc_path)

    # Search by png images
    for fname in sorted(os.listdir(rc_path)):
        files.append(TEMPLATE_QRC_FILE.format(fname=fname))

    # Join parts
    qrc_content = (TEMPLATE_QRC_HEADER.format(resource_prefix=resource_prefix)
                   + '\n'.join(files)
                   + TEMPLATE_QRC_FOOTER.format(style_prefix=style_prefix))

    _logger.info("Writing in: %s", qrc_path)

    # Write qrc file
    with open(qrc_path, 'w') as fh:
        fh.write(qrc_content)


def get_rc_links_from_scss(pattern=r"\/.*\.png"):
    """
    Get all rc links from scss file returning the list of unique links.

    Args:
        pattern (str): regex pattern to find the links.

    Returns:
        list(str): list of unique links found.
    """

    with open(STYLES_SCSS_FILEPATH, 'r') as fh:
        data = fh.read()

    lines = data.split("\n")
    compiled_exp = re.compile('(' + pattern + ')')

    rc_list = []

    for line in lines:
        match = re.search(compiled_exp, line)
        if match:
            rc_list.append(match.group(1))

    rc_list = list(set(rc_list))

    return rc_list



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""QRainbowStyle default palette."""

# Standard library imports
from collections import OrderedDict

# package imports
from qrainbowstyle.colorsystem import *


class BasePalette:
    """Base class for palettes."""

    # Color
    COLOR_BACKGROUND_1 = ''
    COLOR_BACKGROUND_2 = ''
    COLOR_BACKGROUND_3 = ''
    COLOR_BACKGROUND_4 = ''
    COLOR_BACKGROUND_5 = ''
    COLOR_BACKGROUND_6 = ''

    COLOR_TEXT_1 = ''
    COLOR_TEXT_2 = ''
    COLOR_TEXT_3 = ''
    COLOR_TEXT_4 = ''

    COLOR_ACCENT_1 = ''
    COLOR_ACCENT_2 = ''
    COLOR_ACCENT_3 = ''
    COLOR_ACCENT_4 = ''
    COLOR_ACCENT_5 = ''

    OPACITY_TOOLTIP = 0

    # Size
    SIZE_BORDER_RADIUS = '4px'

    # Borders
    BORDER_1 = '1px solid $COLOR_BACKGROUND_1'
    BORDER_2 = '1px solid $COLOR_BACKGROUND_4'
    BORDER_3 = '1px solid $COLOR_BACKGROUND_6'

    BORDER_SELECTION_3 = '1px solid $COLOR_ACCENT_3'
    BORDER_SELECTION_2 = '1px solid $COLOR_ACCENT_2'
    BORDER_SELECTION_1 = '1px solid $COLOR_ACCENT_1'

    TITLE_BAR_BACKGROUND_COLOR = COLOR_ACCENT_3
    TITLE_BAR_BUTTONS_HOVER_COLOR = COLOR_ACCENT_4
    TITLE_BAR_BUTTONS_DISABLED_COLOR = COLOR_ACCENT_1
    TITLE_BAR_TEXT_COLOR = COLOR_TEXT_1

    # Paths
    PATH_RESOURCES = "':/qss_icons'"

    @classmethod
    def to_dict(cls, colors_only=False):
        """Convert variables to dictionary."""
        order = [
            'COLOR_BACKGROUND_6',
            'COLOR_BACKGROUND_5',
            'COLOR_BACKGROUND_4',
            'COLOR_BACKGROUND_2',
            'COLOR_BACKGROUND_3',
            'COLOR_BACKGROUND_1',
            'COLOR_TEXT_1',
            'COLOR_TEXT_2',
            'COLOR_TEXT_3',
            'COLOR_TEXT_4',
            'COLOR_ACCENT_1',
            'COLOR_ACCENT_2',
            'COLOR_ACCENT_3',
            'COLOR_ACCENT_4',
            'OPACITY_TOOLTIP',
            'SIZE_BORDER_RADIUS',
            'BORDER_1',
            'BORDER_2',
            'BORDER_3',
            'BORDER_SELECTION_3',
            'BORDER_SELECTION_2',
            'BORDER_SELECTION_1',
            'TITLE_BAR_BACKGROUND_COLOR',
            'TITLE_BAR_BUTTONS_HOVER_COLOR',
            'TITLE_BAR_BUTTONS_DISABLED_COLOR',
            'TITLE_BAR_TEXT_COLOR',
            'PATH_RESOURCES',
        ]
        dic = OrderedDict()
        for var in order:
            value = getattr(cls, var)

            if colors_only:
                if not var.startswith('COLOR'):
                    value = None

            if value:
                dic[var] = value

        return dic

    @classmethod
    def color_palette(cls):
        """Return the ordered colored palette dictionary."""
        return cls.to_dict(colors_only=True)


class Oceanic(BasePalette):
    """Theme variables."""
    COLOR_BACKGROUND_1 = "#263238"
    COLOR_BACKGROUND_2 = "#2f4048"
    COLOR_BACKGROUND_3 = "#34474f"
    COLOR_BACKGROUND_4 = "#394d57"
    COLOR_BACKGROUND_5 = "#3d545f"
    COLOR_BACKGROUND_6 = "#425b67"

    COLOR_TEXT_1 = Gray.B130
    COLOR_TEXT_2 = Gray.B110
    COLOR_TEXT_3 = Gray.B90
    COLOR_TEXT_4 = Gray.B80

    COLOR_ACCENT_1 = "#0a4542"
    COLOR_ACCENT_2 = "#136460"
    COLOR_ACCENT_3 = "#097D74"
    COLOR_ACCENT_4 = "#56BFBA"
    COLOR_ACCENT_5 = "#C4D6DB"

    TITLE_BAR_BACKGROUND_COLOR = COLOR_ACCENT_3
    TITLE_BAR_BUTTONS_HOVER_COLOR = COLOR_ACCENT_4
    TITLE_BAR_BUTTONS_DISABLED_COLOR = COLOR_ACCENT_1
    TITLE_BAR_TEXT_COLOR = COLOR_TEXT_1

    OPACITY_TOOLTIP = 230


class QDarkStyle(BasePalette):
    COLOR_BACKGROUND_1 = "#19232d"
    COLOR_BACKGROUND_2 = "#27323c"
    COLOR_BACKGROUND_3 = "#35414b"
    COLOR_BACKGROUND_4 = "#3e4b55"
    COLOR_BACKGROUND_5 = "#47555f"
    COLOR_BACKGROUND_6 = "#505f69"

    COLOR_TEXT_1 = Gray.B130
    COLOR_TEXT_2 = Gray.B110
    COLOR_TEXT_3 = Gray.B90
    COLOR_TEXT_4 = Gray.B80

    COLOR_ACCENT_1 = "#14506e"
    COLOR_ACCENT_2 = "#145f87"
    COLOR_ACCENT_3 = "#146998"
    COLOR_ACCENT_4 = "#1478b1"
    COLOR_ACCENT_5 = "#148cd2"

    TITLE_BAR_BACKGROUND_COLOR = COLOR_ACCENT_3
    TITLE_BAR_BUTTONS_HOVER_COLOR = COLOR_ACCENT_4
    TITLE_BAR_BUTTONS_DISABLED_COLOR = COLOR_ACCENT_1
    TITLE_BAR_TEXT_COLOR = COLOR_TEXT_1

    OPACITY_TOOLTIP = 230


class DarkOrange(BasePalette):
    COLOR_BACKGROUND_1 = Gray.B10
    COLOR_BACKGROUND_2 = Gray.B20
    COLOR_BACKGROUND_3 = Gray.B30
    COLOR_BACKGROUND_4 = Gray.B40
    COLOR_BACKGROUND_5 = Gray.B50
    COLOR_BACKGROUND_6 = Gray.B60

    COLOR_TEXT_1 = Gray.B130
    COLOR_TEXT_2 = Gray.B110
    COLOR_TEXT_3 = Gray.B90
    COLOR_TEXT_4 = Gray.B80

    COLOR_ACCENT_1 = "#ce4b01"
    COLOR_ACCENT_2 = "#d66522"
    COLOR_ACCENT_3 = "#de8044"
    COLOR_ACCENT_4 = "#e79b65"
    COLOR_ACCENT_5 = "#efb587"

    TITLE_BAR_BACKGROUND_COLOR = COLOR_ACCENT_3
    TITLE_BAR_BUTTONS_HOVER_COLOR = COLOR_ACCENT_4
    TITLE_BAR_BUTTONS_DISABLED_COLOR = COLOR_ACCENT_1
    TITLE_BAR_TEXT_COLOR = COLOR_TEXT_1

    OPACITY_TOOLTIP = 230


class LightOrange(BasePalette):
    COLOR_BACKGROUND_1 = Gray.B140
    COLOR_BACKGROUND_2 = Gray.B130
    COLOR_BACKGROUND_3 = Gray.B120
    COLOR_BACKGROUND_4 = Gray.B110
    COLOR_BACKGROUND_5 = Gray.B100
    COLOR_BACKGROUND_6 = Gray.B90

    COLOR_TEXT_1 = Gray.B10
    COLOR_TEXT_2 = Gray.B20
    COLOR_TEXT_3 = Gray.B50
    COLOR_TEXT_4 = Gray.B70

    COLOR_ACCENT_1 = "#ce4b01"
    COLOR_ACCENT_2 = "#d66522"
    COLOR_ACCENT_3 = "#de8044"
    COLOR_ACCENT_4 = "#e79b65"
    COLOR_ACCENT_5 = "#efb587"

    TITLE_BAR_BACKGROUND_COLOR = COLOR_ACCENT_3
    TITLE_BAR_BUTTONS_HOVER_COLOR = COLOR_ACCENT_4
    TITLE_BAR_BUTTONS_DISABLED_COLOR = COLOR_ACCENT_1
    TITLE_BAR_TEXT_COLOR = COLOR_TEXT_1

    OPACITY_TOOLTIP = 230


class QDarkStyle3Light(BasePalette):
    # Color
    COLOR_BACKGROUND_1 = Gray.B140
    COLOR_BACKGROUND_2 = Gray.B130
    COLOR_BACKGROUND_3 = Gray.B120
    COLOR_BACKGROUND_4 = Gray.B110
    COLOR_BACKGROUND_5 = Gray.B100
    COLOR_BACKGROUND_6 = Gray.B90

    COLOR_TEXT_1 = Gray.B10
    COLOR_TEXT_2 = Gray.B20
    COLOR_TEXT_3 = Gray.B50
    COLOR_TEXT_4 = Gray.B70

    COLOR_ACCENT_1 = Blue.B130
    COLOR_ACCENT_2 = Blue.B100
    COLOR_ACCENT_3 = Blue.B90
    COLOR_ACCENT_4 = Blue.B80
    COLOR_ACCENT_5 = Blue.B70

    TITLE_BAR_BACKGROUND_COLOR = COLOR_ACCENT_3
    TITLE_BAR_BUTTONS_HOVER_COLOR = COLOR_ACCENT_4
    TITLE_BAR_BUTTONS_DISABLED_COLOR = COLOR_ACCENT_1
    TITLE_BAR_TEXT_COLOR = COLOR_TEXT_1

    OPACITY_TOOLTIP = 230


class QDarkStyle3(BasePalette):
    # Color
    COLOR_BACKGROUND_1 = Gray.B10
    COLOR_BACKGROUND_2 = Gray.B20
    COLOR_BACKGROUND_3 = Gray.B30
    COLOR_BACKGROUND_4 = Gray.B40
    COLOR_BACKGROUND_5 = Gray.B50
    COLOR_BACKGROUND_6 = Gray.B60

    COLOR_TEXT_1 = Gray.B130
    COLOR_TEXT_2 = Gray.B110
    COLOR_TEXT_3 = Gray.B90
    COLOR_TEXT_4 = Gray.B80

    COLOR_ACCENT_1 = Blue.B20
    COLOR_ACCENT_2 = Blue.B40
    COLOR_ACCENT_3 = Blue.B50
    COLOR_ACCENT_4 = Blue.B70
    COLOR_ACCENT_5 = Blue.B80

    TITLE_BAR_BACKGROUND_COLOR = COLOR_ACCENT_3
    TITLE_BAR_BUTTONS_HOVER_COLOR = COLOR_ACCENT_4
    TITLE_BAR_BUTTONS_DISABLED_COLOR = COLOR_ACCENT_1
    TITLE_BAR_TEXT_COLOR = COLOR_TEXT_1

    OPACITY_TOOLTIP = 230



from qtpy.QtWidgets import QFrame, QMenu, QHBoxLayout, QLabel, QSizePolicy
from qtpy.QtCore import Signal, QPoint, QMetaObject, Slot, Qt, QRect
from qtpy.QtGui import QPalette, QPixmap, QIcon

from .Buttons import ButtonsWidget, AppLogo, MenuButton

import qrainbowstyle


class Titlebar(QFrame):
    """Titlebar for frameless windows."""

    minimizeClicked = Signal()
    maximizeClicked = Signal()
    restoreClicked = Signal()
    closeClicked = Signal()

    def __init__(self, parent=None):
        super(Titlebar, self).__init__(parent)
        self.setObjectName("titlebar")
        self.setMouseTracking(True)

        self.menus = []

        self.setAutoFillBackground(True)
        self.setFixedHeight(30)
        self.setContentsMargins(0, 0, 0, 0)
        self.setBackgroundRole(QPalette.Highlight)

        self.layout = QHBoxLayout(self)
        self.layout.setAlignment(Qt.AlignVCenter)
        self.layout.setAlignment(Qt.AlignLeft)
        self.layout.setContentsMargins(8, 0, 0, 0)
        self.layout.setSpacing(0)
        self.setLayout(self.layout)

        self.appLogoLabel = AppLogo(self)
        if qrainbowstyle.APP_ICON_PATH:
            self.appLogoLabel.setPixmap(QPixmap(qrainbowstyle.APP_ICON_PATH))
            self.layout.setContentsMargins(2, 0, 0, 0)
        self.layout.addWidget(self.appLogoLabel)
        if qrainbowstyle.ALIGN_BUTTONS_LEFT:
            self.appLogoLabel.setVisible(False)
            self.layout.setContentsMargins(8, 0, 0, 0)

        self.layout.insertStretch(50)
        self.buttonsWidget = ButtonsWidget(self)
        if qrainbowstyle.ALIGN_BUTTONS_LEFT:
            self.layout.insertWidget(0, self.buttonsWidget)
        else:
            self.layout.addWidget(self.buttonsWidget)

        # auto connect signals
        QMetaObject.connectSlotsByName(self)
        if self.window().parent() is not None:
            self.buttonsWidget.btnRestore.setVisible(False)
            self.buttonsWidget.btnMaximize.setVisible(False)
            self.buttonsWidget.btnMinimize.setVisible(False)

    def setWindowIcon(self, icon: QIcon):
        self.appLogoLabel.setPixmap(icon.pixmap())

    # connecting buttons signals
    @Slot()
    def on_btnClose_clicked(self):
        self.closeClicked.emit()

    @Slot()
    def on_btnRestore_clicked(self):
        self.showRestoreButton(False)
        self.showMaximizeButton(True)
        self.window().showNormal()
        self.restoreClicked.emit()

    @Slot()
    def on_btnMaximize_clicked(self):
        if qrainbowstyle.USE_DARWIN_BUTTONS:
            if self.window().isMaximized():
                self.window().showNormal()
            else:
                self.window().showMaximized()
        else:
            self.showRestoreButton(True)
            self.showMaximizeButton(False)
            self.window().showMaximized()
        self.maximizeClicked.emit()

    @Slot()
    def on_btnMinimize_clicked(self):
        self.window().showMinimized()

    def showLogo(self, value: bool):
        """Show or hide app logo label"""
        self.appLogoLabel.setVisible(value)

    def showRestoreButton(self, value):
        if self.window().parent() is None:
            self.buttonsWidget.btnRestore.setVisible(value)

    def showMaximizeButton(self, value):
        if self.window().parent() is None:
            self.buttonsWidget.btnMaximize.setVisible(value)

    def showMinimizeButton(self, value):
        if self.window().parent() is None:
            self.buttonsWidget.btnMinimize.setVisible(value)

    def addMenu(self, menu: QMenu):
        menuButton = MenuButton(self)
        menuButton.setMenu(menu)
        menuButton.setText(menu.title())
        self.layout.insertWidget(len(self.menus) + 1, menuButton)
        self.menus.append(menuButton)

    def setTitlebarHeight(self, height: int):
        self.setFixedHeight(height)

    def mouseOverTitlebar(self, x, y):
        if self.childAt(QPoint(x, y)):
            return False
        else:
            return QRect(self.appLogoLabel.width(), 0,
                         self.width() - self.appLogoLabel.width(),
                         self.height()).contains(QPoint(x, y))



import qrainbowstyle
from qtpy.QtWidgets import QApplication, QWidget, QVBoxLayout, QSizePolicy, QDialog
from qtpy.QtGui import QIcon
from qtpy.QtCore import Qt, QMetaObject, QEvent, QSize, Signal, Slot

from .Titlebar import Titlebar


class FramelessWindowBase(QDialog):
    closeClicked = Signal()

    def __init__(self, parent):
        super(FramelessWindowBase, self).__init__(parent)
        self.__rect = QApplication.instance().desktop().availableGeometry(self)

        self.__resizingEnabled = True
        self.__contentWidgets = []

        self.setMouseTracking(True)
        self.setAttribute(Qt.WA_NoSystemBackground)
        super().setContentsMargins(0, 0, 0, 0)
        super().setWindowFlags(Qt.Window
                               | Qt.FramelessWindowHint
                               | Qt.WindowSystemMenuHint
                               | Qt.WindowMinimizeButtonHint
                               | Qt.WindowMaximizeButtonHint
                               | Qt.WindowCloseButtonHint)

        self.__centralWidget = QWidget(self)
        self.__centralWidget.setObjectName("__centralWidget")
        self.__centralWidget.setContentsMargins(0, 0, 0, 0)
        self.__centralWidget.setMouseTracking(True)

        self.__centralLayout = QVBoxLayout(self.__centralWidget)
        self.__centralLayout.setAlignment(Qt.AlignBottom)
        self.__centralLayout.setContentsMargins(0, 0, 0, 0)
        self.__centralLayout.setSpacing(0)

        self.__bar = Titlebar(self)
        self.__bar.showRestoreButton(False)
        self.__bar.showMinimizeButton(False)
        self.__bar.showMaximizeButton(False)
        self.__centralLayout.addWidget(self.__bar)
        self.__centralLayout.setAlignment(self.__bar, Qt.AlignVCenter)

        self.__contentWidget = QWidget(self)
        self.__contentWidget.setObjectName("__contentWidget")
        self.__contentWidget.setSizePolicy(
            QSizePolicy.Expanding,
            QSizePolicy.Expanding)
        self.__contentWidget.setContentsMargins(2, 0, 2, 2)
        self.__contentWidgetLayout = QVBoxLayout(self.__contentWidget)
        self.__contentWidgetLayout.setContentsMargins(0, 0, 0, 0)
        self.__centralLayout.addWidget(self.__contentWidget)

        self.__centralWidget.setLayout(self.__centralLayout)
        self.__contentWidget.setAutoFillBackground(True)

        self.__bar.closeClicked.connect(self.closeClicked.emit)
        self.closeClicked.connect(self.close)

        self.__main_layout = QVBoxLayout(self)
        self.__main_layout.setContentsMargins(0, 0, 0, 0)
        self.setLayout(self.__main_layout)
        self.__main_layout.addWidget(self.__centralWidget)

        QMetaObject.connectSlotsByName(self)
        self.showSizeControl(True)
        self.resize(QSize(self.__rect.width() / 2, self.__rect.height() / 2))

    def setContentsMargins(self, left, top, right, bottom):
        self.__contentWidgetLayout.setContentsMargins(left, top, right, bottom)

    def showWindowShadow(self, value: bool):
        pass

    def setEdgeSnapping(self, value: bool):
        pass

    def setResizingEnabled(self, value: bool):
        """Enable window resizing

        Args:
            value (bool): Enable or disable window resizing
        """
        self.__resizingEnabled = value

    def addMenu(self, menu):
        self.__bar.addMenu(menu)

    def titlebar(self):
        return self.__bar

    def showSizeControl(self, value: bool):
        self.__bar.showMaximizeButton(value)
        self.__bar.showMinimizeButton(value)

    def isResizingEnabled(self) -> bool:
        """Return if window allows resizing

        Returns:
            value (bool): Window allow resizing.
        """
        return self.__resizingEnabled

    def setTitlebarHeight(self, height: int):
        """Set titlebar height.

        Args:
            height (int): Titlebar height.
        """
        self.__bar.setTitlebarHeight(height)

    def addContentWidget(self, widget: QWidget):
        """Add master widget to window.

        Args:
            widget (QWidget): Content widget.
        """
        self.__contentWidgets.append(widget)
        self.__contentWidgetLayout.addWidget(widget)

    def insertContentWidget(self, index, widget: QWidget):
        """Insert master widget to window at pos.

        Args:
            index (int): Index
            widget (QWidget): Content widget.
        """
        self.__contentWidgets.insert(index, widget)
        self.__contentWidgetLayout.insertWidget(index, widget)

    def setWindowIcon(self, icon: QIcon) -> None:
        self.__bar.setWindowIcon(icon)
        super().setWindowIcon(icon)

    def changeEvent(self, event: QEvent) -> None:
        if event.type() == QEvent.WindowStateChange and not qrainbowstyle.USE_DARWIN_BUTTONS:
            if self.isMaximized():
                self.__bar.showRestoreButton(True)
                self.__bar.showMaximizeButton(False)
            else:
                self.__bar.showRestoreButton(False)
                self.__bar.showMaximizeButton(True)
        return super().changeEvent(event)



from qrainbowstyle.widgets.QtWaitingSpinner.pyqtspinner.spinner import WaitingSpinner
from qrainbowstyle.widgets.GoogleMapsWidget.MapsWidget import GoogleMapsView, OpenStreetMapsView
from qrainbowstyle.widgets.PythonQtWidgets.qtwidgets.palette.palette import (StylePickerGrid, StylePickerVertical,
                                                                             StylePickerHorizontal)
from qrainbowstyle.widgets.QRoundProgressBar.qroundprogressbar import QRoundProgressBar



"""
The MIT License (MIT)

Copyright (c) 2017 fbjorn

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

import sys

from qtpy.QtCore import Qt
from qtpy.QtWidgets import QWidget, QGridLayout, QGroupBox, QHBoxLayout, QSpinBox, QDoubleSpinBox, QPushButton, QLabel, QColorDialog, QApplication, QMessageBox

from pyqtspinner.spinner import WaitingSpinner

import qrainbowstyle
import qrainbowstyle.widgets
import qrainbowstyle.windows


class Demo(QWidget):
    sb_roundness = None
    sb_opacity = None
    sb_fadeperc = None
    sb_lines = None
    sb_line_length = None
    sb_line_width = None
    sb_inner_radius = None
    sb_rev_s = None

    btn_start = None
    btn_stop = None
    btn_pick_color = None

    spinner = None

    def __init__(self):
        super().__init__()
        self.init_ui()

    def init_ui(self):
        grid = QGridLayout()
        groupbox1 = QGroupBox()
        groupbox1_layout = QHBoxLayout()
        groupbox2 = QGroupBox()
        groupbox2_layout = QGridLayout()
        button_hbox = QHBoxLayout()
        self.setLayout(grid)
        self.setWindowTitle("QtWaitingSpinner Creator")
        self.setWindowFlags(Qt.Dialog)

        # SPINNER
        self.spinner = WaitingSpinner(self)

        # Spinboxes
        self.sb_roundness = QDoubleSpinBox()
        self.sb_opacity = QDoubleSpinBox()
        self.sb_fadeperc = QDoubleSpinBox()
        self.sb_lines = QSpinBox()
        self.sb_line_length = QDoubleSpinBox()
        self.sb_line_width = QDoubleSpinBox()
        self.sb_inner_radius = QDoubleSpinBox()
        self.sb_rev_s = QDoubleSpinBox()

        # set spinbox default values
        self.sb_roundness.setValue(70)
        self.sb_roundness.setRange(0, 9999)
        self.sb_opacity.setValue(15)
        self.sb_opacity.setRange(0, 9999)
        self.sb_fadeperc.setValue(70)
        self.sb_fadeperc.setRange(0, 9999)
        self.sb_lines.setValue(12)
        self.sb_lines.setRange(1, 9999)
        self.sb_line_length.setValue(10)
        self.sb_line_length.setRange(0, 9999)
        self.sb_line_width.setValue(5)
        self.sb_line_width.setRange(0, 9999)
        self.sb_inner_radius.setValue(10)
        self.sb_inner_radius.setRange(0, 9999)
        self.sb_rev_s.setValue(1)
        self.sb_rev_s.setRange(0.1, 9999)

        # Buttons
        self.btn_start = QPushButton("Start")
        self.btn_stop = QPushButton("Stop")
        self.btn_pick_color = QPushButton("Pick Color")
        self.btn_show_init = QPushButton("Show init args")
        self.btn_fade_in = QPushButton("Fade in")
        self.btn_fade_out = QPushButton("Fade out")

        # Connects
        self.sb_roundness.valueChanged.connect(self.set_roundness)
        self.sb_opacity.valueChanged.connect(self.set_opacity)
        self.sb_fadeperc.valueChanged.connect(self.set_fadeperc)
        self.sb_lines.valueChanged.connect(self.set_lines)
        self.sb_line_length.valueChanged.connect(self.set_line_length)
        self.sb_line_width.valueChanged.connect(self.set_line_width)
        self.sb_inner_radius.valueChanged.connect(self.set_inner_radius)
        self.sb_rev_s.valueChanged.connect(self.set_rev_s)

        # qrainbowstyle
        self.picker = qrainbowstyle.widgets.StylePickerGrid(self)

        self.btn_start.clicked.connect(self.spinner_start)
        self.btn_stop.clicked.connect(self.spinner_stop)
        self.btn_pick_color.clicked.connect(self.show_color_picker)
        self.btn_show_init.clicked.connect(self.show_init_args)
        self.btn_fade_in.clicked.connect(self.fadeIn)
        self.btn_fade_out.clicked.connect(self.fadeOut)

        # Layout adds
        groupbox1_layout.addWidget(self.spinner)
        groupbox1.setLayout(groupbox1_layout)

        groupbox2_layout.addWidget(QLabel("Roundness:"), *(1, 1))
        groupbox2_layout.addWidget(self.sb_roundness, *(1, 2))
        groupbox2_layout.addWidget(QLabel("Opacity:"), *(2, 1))
        groupbox2_layout.addWidget(self.sb_opacity, *(2, 2))
        groupbox2_layout.addWidget(QLabel("Fade Perc:"), *(3, 1))
        groupbox2_layout.addWidget(self.sb_fadeperc, *(3, 2))
        groupbox2_layout.addWidget(QLabel("Lines:"), *(4, 1))
        groupbox2_layout.addWidget(self.sb_lines, *(4, 2))
        groupbox2_layout.addWidget(QLabel("Line Length:"), *(5, 1))
        groupbox2_layout.addWidget(self.sb_line_length, *(5, 2))
        groupbox2_layout.addWidget(QLabel("Line Width:"), *(6, 1))
        groupbox2_layout.addWidget(self.sb_line_width, *(6, 2))
        groupbox2_layout.addWidget(QLabel("Inner Radius:"), *(7, 1))
        groupbox2_layout.addWidget(self.sb_inner_radius, *(7, 2))
        groupbox2_layout.addWidget(QLabel("Rev/s:"), *(8, 1))
        groupbox2_layout.addWidget(self.sb_rev_s, *(8, 2))
        groupbox2_layout.addWidget(QLabel("Style: "), *(9, 1))
        groupbox2_layout.addWidget(self.picker, *(9, 2))

        groupbox2.setLayout(groupbox2_layout)

        button_hbox.addWidget(self.btn_start)
        button_hbox.addWidget(self.btn_stop)
        button_hbox.addWidget(self.btn_pick_color)
        button_hbox.addWidget(self.btn_show_init)
        button_hbox.addWidget(self.btn_fade_in)
        button_hbox.addWidget(self.btn_fade_out)

        grid.addWidget(groupbox1, *(1, 1))
        grid.addWidget(groupbox2, *(1, 2))
        grid.addLayout(button_hbox, *(2, 1))

        self.spinner.start()

    def set_roundness(self):
        self.spinner.setRoundness(self.sb_roundness.value())

    def set_opacity(self):
        self.spinner.setMinimumTrailOpacity(self.sb_opacity.value())

    def set_fadeperc(self):
        self.spinner.setTrailFadePercentage(self.sb_fadeperc.value())

    def set_lines(self):
        self.spinner.setNumberOfLines(self.sb_lines.value())

    def set_line_length(self):
        self.spinner.setLineLength(self.sb_line_length.value())

    def set_line_width(self):
        self.spinner.setLineWidth(self.sb_line_width.value())

    def set_inner_radius(self):
        self.spinner.setInnerRadius(self.sb_inner_radius.value())

    def set_rev_s(self):
        self.spinner.setRevolutionsPerSecond(self.sb_rev_s.value())

    def spinner_start(self):
        self.spinner.start()

    def spinner_stop(self):
        self.spinner.stop()

    def show_color_picker(self):
        self.spinner.setColor(QColorDialog.getColor())

    def show_init_args(self):
        text = (
            'WaitingSpinner(\n'
            '    parent,\n'
            '    roundness={}, opacity={},\n'
            '    fade={}, radius={}, lines={},\n'
            '    line_length={}, line_width={},\n'
            '    speed={}, color={}\n'
            ')\n'
        ).format(
            self.sb_roundness.value(), self.sb_opacity.value(),
            self.sb_fadeperc.value(), self.sb_inner_radius.value(),
            self.sb_lines.value(), self.sb_line_length.value(),
            self.sb_line_width.value(), self.sb_rev_s.value(),
            self.spinner.color.getRgb()[:3]
        )

        msg_box = QMessageBox(text=text)
        msg_box.setWindowTitle('Text was copied to clipboard')
        cb = QApplication.clipboard()
        cb.clear(mode=cb.Clipboard)
        cb.setText(text, mode=cb.Clipboard)
        print(text)
        msg_box.exec_()

    def fadeIn(self):
        self.spinner.fadeIn()

    def fadeOut(self):
        self.spinner.fadeOut()


if __name__ == '__main__':
    app = QApplication(sys.argv)
    app.setStyleSheet(qrainbowstyle.load_stylesheet())
    win = qrainbowstyle.windows.FramelessWindow()
    main = Demo()
    win.addContentWidget(main)
    win.show()
    sys.exit(app.exec())



import logging
import traceback

from qtpy.QtCore import QtInfoMsg, QtFatalMsg, QtCriticalMsg, QtWarningMsg


class OutputLogger:
    """Logging cocfiguration class"""

    def __init__(self):
        super(OutputLogger, self).__init__()
        logging.basicConfig(
            level=logging.NOTSET,
            format="%(asctime)s [%(threadName)s] [%(name)s] [%(levelname)s] %(message)s",
            handlers=[logging.StreamHandler()])
        logging.getLogger(self.__class__.__name__).debug("Logger enabled.")


def qt_message_handler(mode, context, message):
    logger = logging.getLogger("QT Logger")
    """Qt errors handler"""
    if mode == QtInfoMsg:
        mode = 20
    elif mode == QtWarningMsg:
        mode = 30
    elif mode == QtCriticalMsg:
        mode = 40
    elif mode == QtFatalMsg:
        mode = 50
    else:
        mode = 20
    logger.log(mode, "(%s: %s): %s" % (context.file, context.line, message))


def catch_exceptions(t, val, tb):
    """Replaces sys.excepthook to catch and log warnings and errors"""
    trace = "".join(traceback.format_exception(t, val, tb))
    logging.getLogger().error(trace)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#######################################################
#
# Copyright 2017 Pete Alexandrou
#
# Ported to Python from the original works in C++ by:
#
#     Sintegrial Technologies (c) 2015
#     https://sourceforge.net/projects/qroundprogressbar
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#######################################################

import operator
from enum import Enum

from qtpy.QtCore import Slot, QPointF, Qt, QRectF
from qtpy.QtGui import (QPalette, QConicalGradient, QGradient, QRadialGradient,
                        QFontMetricsF, QFont, QPainter, QPen, QPainterPath, QImage,
                        QPaintEvent)
from qtpy.QtWidgets import QWidget


class QRoundProgressBar(QWidget):
    # CONSTANTS

    PositionLeft = 180
    PositionTop = 90
    PositionRight = 0
    PositionBottom = -90

    # CONSTRUCTOR ---------------------------------------------------

    def __init__(self, parent=None):
        super(QRoundProgressBar, self).__init__(parent)
        self.m_min = 0
        self.m_max = 100
        self.m_value = 25
        self.m_nullPosition = QRoundProgressBar.PositionTop
        self.m_barStyle = self.BarStyle.DONUT
        self.m_outlinePenWidth = 1
        self.m_dataPenWidth = 1
        self.m_rebuildBrush = False
        self.m_format = '%p%'
        self.m_decimals = 1
        self.m_updateFlags = self.UpdateFlags.PERCENT
        self.m_gradientData = None

    # ENUMS ---------------------------------------------------------

    class BarStyle(Enum):
        DONUT = 0,
        PIE = 1,
        LINE = 2,
        EXPAND = 3

    class UpdateFlags(Enum):
        VALUE = 0,
        PERCENT = 1,
        MAX = 2

    # GETTERS -------------------------------------------------------

    def minimum(self):
        return self.m_min

    def maximum(self):
        return self.m_max

    # SETTERS -------------------------------------------------------

    def setNullPosition(self, position: float):
        if position != self.m_nullPosition:
            self.m_nullPosition = position
            self.m_rebuildBrush = True
            self.update()

    def setBarStyle(self, style: BarStyle):
        if style != self.m_barStyle:
            self.m_barStyle = style
            self.m_rebuildBrush = True
            self.update()

    def setOutlinePenWidth(self, width: float):
        if width != self.m_outlinePenWidth:
            self.m_outlinePenWidth = width
            self.update()

    def setDataPenWidth(self, width: float):
        if width != self.m_dataPenWidth:
            self.m_dataPenWidth = width
            self.update()

    def setDataColors(self, stopPoints: list):
        if stopPoints != self.m_gradientData:
            self.m_gradientData = stopPoints
            self.m_rebuildBrush = True
            self.update()

    def setFormat(self, val: str):
        if val != self.m_format:
            self.m_format = val
            self.valueFormatChanged()

    def resetFormat(self):
        self.m_format = None
        self.valueFormatChanged()

    def setDecimals(self, count: int):
        if count >= 0 and count != self.m_decimals:
            self.m_decimals = count
            self.valueFormatChanged()

    # SLOTS ---------------------------------------------------------

    @Slot(float, float)
    def setRange(self, minval: float, maxval: float):
        self.m_min = minval
        self.m_max = maxval
        if self.m_max < self.m_min:
            self.m_min = maxval
            self.m_max = minval
        if self.m_value < self.m_min:
            self.m_value = self.m_min
        elif self.m_value > self.m_max:
            self.m_value = self.m_max
        self.m_rebuildBrush = True
        self.update()

    @Slot(float)
    def setMinimum(self, val: float):
        self.setRange(val, self.m_max)

    @Slot(float)
    def setMaximum(self, val: float):
        self.setRange(self.m_min, val)

    @Slot(int)
    def setValue(self, val: int):
        if self.m_value != val:
            if val < self.m_min:
                self.m_value = self.m_min
            elif val > self.m_max:
                self.m_value = self.m_max
            else:
                self.m_value = val
            self.update()

    # PAINTING ------------------------------------------------------

    def paintEvent(self, event: QPaintEvent):
        outerRadius = min(self.width(), self.height())
        baseRect = QRectF(1, 1, outerRadius - 2, outerRadius - 2)
        buffer = QImage(outerRadius, outerRadius, QImage.Format_ARGB32_Premultiplied)
        p = QPainter(buffer)
        p.setRenderHint(QPainter.Antialiasing)
        self.rebuildDataBrushIfNeeded()
        self.drawBackground(p, buffer.rect())
        self.drawBase(p, baseRect)
        if self.m_value > 0:
            delta = (self.m_max - self.m_min) / (self.m_value - self.m_min)
        else:
            delta = 0
        self.drawValue(p, baseRect, self.m_value, delta)
        innerRect, innerRadius = self.calculateInnerRect(outerRadius)
        self.drawInnerBackground(p, innerRect)
        self.drawText(p, innerRect, innerRadius, self.m_value)
        p.end()
        painter = QPainter(self)
        painter.fillRect(baseRect, self.palette().window())
        painter.drawImage(0, 0, buffer)

    def drawBackground(self, p: QPainter, baseRect: QRectF):
        p.fillRect(baseRect, self.palette().window())

    def drawBase(self, p: QPainter, baseRect: QRectF):
        if self.m_barStyle == self.BarStyle.DONUT:
            p.setPen(QPen(self.palette().shadow().color(), self.m_outlinePenWidth))
            p.setBrush(self.palette().base())
            p.drawEllipse(baseRect)
        elif self.m_barStyle == self.BarStyle.LINE:
            p.setPen(QPen(self.palette().base().color(), self.m_outlinePenWidth))
            p.setBrush(Qt.NoBrush)
            p.drawEllipse(baseRect.adjusted(self.m_outlinePenWidth / 2, self.m_outlinePenWidth / 2,
                                            -self.m_outlinePenWidth / 2, -self.m_outlinePenWidth / 2))
        elif self.m_barStyle in (self.BarStyle.PIE, self.BarStyle.EXPAND):
            p.setPen(QPen(self.palette().base().color(), self.m_outlinePenWidth))
            p.setBrush(self.palette().base())
            p.drawEllipse(baseRect)

    def drawValue(self, p: QPainter, baseRect: QRectF, value: float, delta: float):
        if value == self.m_min:
            return
        if self.m_barStyle == self.BarStyle.EXPAND:
            p.setBrush(self.palette().highlight())
            p.setPen(QPen(self.palette().shadow().color(), self.m_dataPenWidth))
            radius = (baseRect.height() / 2) / delta
            p.drawEllipse(baseRect.center(), radius, radius)
            return
        if self.m_barStyle == self.BarStyle.LINE:
            p.setPen(QPen(self.palette().highlight().color(), self.m_dataPenWidth))
            p.setBrush(Qt.NoBrush)
            if value == self.m_max:
                p.drawEllipse(baseRect.adjusted(self.m_outlinePenWidth / 2, self.m_outlinePenWidth / 2,
                                                -self.m_outlinePenWidth / 2, -self.m_outlinePenWidth / 2))
            else:
                arcLength = 360 / delta
                p.drawArc(baseRect.adjusted(self.m_outlinePenWidth / 2, self.m_outlinePenWidth / 2,
                                            -self.m_outlinePenWidth / 2, -self.m_outlinePenWidth / 2),
                          int(self.m_nullPosition * 16),
                          int(-arcLength * 16))
            return
        dataPath = QPainterPath()
        dataPath.setFillRule(Qt.WindingFill)
        if value == self.m_max:
            dataPath.addEllipse(baseRect)
        else:
            arcLength = 360 / delta
            dataPath.moveTo(baseRect.center())
            dataPath.arcTo(baseRect, self.m_nullPosition, -arcLength)
            dataPath.lineTo(baseRect.center())
        p.setBrush(self.palette().highlight())
        p.setPen(QPen(self.palette().shadow().color(), self.m_dataPenWidth))
        p.drawPath(dataPath)

    def calculateInnerRect(self, outerRadius: float):
        if self.m_barStyle in (self.BarStyle.LINE, self.BarStyle.EXPAND):
            innerRadius = outerRadius - self.m_outlinePenWidth
        else:
            innerRadius = outerRadius * 0.75
        delta = (outerRadius - innerRadius) / 2
        innerRect = QRectF(delta, delta, innerRadius, innerRadius)
        return innerRect, innerRadius

    def drawInnerBackground(self, p: QPainter, innerRect: QRectF):
        if self.m_barStyle == self.BarStyle.DONUT:
            p.setBrush(self.palette().base())
            p.drawEllipse(innerRect)

    def drawText(self, p: QPainter, innerRect: QRectF, innerRadius: float, value: float):
        if not self.m_format:
            return
        f = QFont(self.font())
        f.setPixelSize(10)
        fm = QFontMetricsF(f)
        maxWidth = fm.width(self.valueToText(self.m_max))
        delta = innerRadius / maxWidth
        fontSize = f.pixelSize() * delta * 0.75
        f.setPixelSize(int(fontSize))
        p.setFont(f)
        textRect = QRectF(innerRect)
        p.setPen(self.palette().text().color())
        p.drawText(textRect, Qt.AlignCenter, self.valueToText(value))

    def valueToText(self, value: float):
        textToDraw = self.m_format
        if self.m_updateFlags == self.UpdateFlags.VALUE:
            textToDraw = textToDraw.replace('%v', str(round(value, self.m_decimals)))
        if self.m_updateFlags == self.UpdateFlags.PERCENT:
            procent = (value - self.m_min) / (self.m_max - self.m_min) * 100
            textToDraw = textToDraw.replace('%p', str(round(procent, self.m_decimals)))
        if self.m_updateFlags == self.UpdateFlags.MAX:
            textToDraw = textToDraw.replace('%m', str(round(self.m_max - self.m_min + 1, self.m_decimals)))
        return textToDraw

    def valueFormatChanged(self):
        if operator.contains(self.m_format, '%v'):
            self.m_updateFlags = self.UpdateFlags.VALUE
        if operator.contains(self.m_format, '%p'):
            self.m_updateFlags = self.UpdateFlags.PERCENT
        if operator.contains(self.m_format, '%m'):
            self.m_updateFlags = self.UpdateFlags.MAX
        self.update()

    def rebuildDataBrushIfNeeded(self):
        if not self.m_rebuildBrush or not self.m_gradientData or self.m_barStyle == self.BarStyle.LINE:
            return
        self.m_rebuildBrush = False
        p = self.palette()
        if self.m_barStyle == self.BarStyle.EXPAND:
            dataBrush = QRadialGradient(0.5, 0.5, 0.5, 0.5, 0.5)
            dataBrush.setCoordinateMode(QGradient.StretchToDeviceMode)
            for i in range(0, len(self.m_gradientData)):
                dataBrush.setColorAt(self.m_gradientData[i][0], self.m_gradientData[i][1])
            p.setBrush(QPalette.Highlight, dataBrush)
        else:
            dataBrush = QConicalGradient(QPointF(0.5, 0.5), self.m_nullPosition)
            dataBrush.setCoordinateMode(QGradient.StretchToDeviceMode)
            for i in range(0, len(self.m_gradientData)):
                dataBrush.setColorAt(1 - self.m_gradientData[i][0], self.m_gradientData[i][1])
            p.setBrush(QPalette.Highlight, dataBrush)
        self.setPalette(p)



import logging

from qtpy.QtCore import QObject, QRect, Qt, QPoint, QSize, QEvent
from qtpy.QtGui import QMouseEvent, QCursor
from qtpy.QtWidgets import QApplication, QWidget

from .base import FramelessWindowBase


class FramelessWindow(FramelessWindowBase):
    """Frameless window for non-Windows OS like Linux and Darwin.
    Reimplements features:
        - window moving
        - window resizing
        - maximize on double click on titlebar
        - snap to borders
    """

    def __init__(self, parent=None):
        super(FramelessWindow, self).__init__(parent)

        self.__titlebarHeight = 45
        self.__gripsize = 3
        self.__sideGripIgnore = 20
        self.__griprect = None
        self.__hgriprect = None
        self.__vgriprect = None

        self.__moving = False
        self.__move_offset = QPoint()

        self.__resizing = False
        self.__horizontalResizing = False
        self.__verticalResizing = False

        self.installEventFilter(self)
        self.__updateGripRect()
        self.setMouseTracking(True)

    def setMouseTracking(self, flag):
        """Recursively enables mouse tracking for all child widgets.
        This is required because eventFilter does not catch
        hover events.

        Args:
            flag (bool): Enable or disable mouse tracking.
        """
        def recursive_set(parent):
            for child in parent.findChildren(QObject):
                try:
                    child.setMouseTracking(flag)
                except AttributeError:
                    pass
                recursive_set(child)

        QWidget.setMouseTracking(self, flag)
        recursive_set(self)

    def __updateGripRect(self):
        """Update rects for current window geometry."""
        # corner grip
        self.__griprect = QRect(self.width() - self.__gripsize,
                                self.height() - self.__gripsize,
                                self.__gripsize,
                                self.__gripsize)

        # right grip
        self.__hgriprect = QRect(self.width() - self.__gripsize,
                                 self.__titlebarHeight,
                                 self.__gripsize,
                                 self.height() - self.__gripsize - self.__sideGripIgnore - self.__titlebarHeight)

        # bottom grip
        self.__vgriprect = QRect(self.__gripsize,
                                 self.height() - self.__gripsize,
                                 self.width() - self.__gripsize - self.__sideGripIgnore,
                                 self.__gripsize)

    def eventFilter(self, widget, event: QEvent):
        """Handle frameless window events.

        Args:
            widget (QObject): Widget.
            event (QEvent): Event.
        """
        if (hasattr(widget, "window")
                and widget.window() is self):
            self.__updateGripRect()

            if (hasattr(event, "x")
                    and self.titlebar().rect().contains(self.mapFromParent(QCursor.pos()))
                    and self.titlebar().mouseOverTitlebar(event.x(), event.y())):
                # when cursor is over titlebar

                if event.type() == QMouseEvent.MouseButtonPress and event.buttons() == Qt.LeftButton:
                    # if titlebar clicked with left button
                    self.__moving = True
                    margins = self.titlebar().contentsMargins()
                    self.__move_offset = event.pos() + self.titlebar().pos() - QPoint(margins.left(), margins.top())

                elif event.type() == QMouseEvent.MouseMove and event.buttons() == Qt.LeftButton:
                    # if holding left button and moving window
                    if self.__moving:
                        if self.windowState() == Qt.WindowMaximized:
                            self.setWindowState(Qt.WindowNoState)
                            self.titlebar().on_btnRestore_clicked()
                            self.move(event.pos() - self.mapFromParent(event.pos()))
                            self.__moving = False
                        else:
                            self.move(event.globalPos() - self.__move_offset)

                elif event.type() == QMouseEvent.MouseButtonRelease and event.button() == Qt.LeftButton:
                    # if left button released
                    self.__moving = False
                    screen = QApplication.instance().desktop().availableGeometry()
                    if event.globalY() == 0:
                        # snap to top edge
                        pass
                        self.titlebar().on_btnMaximize_clicked()

                    elif event.globalX() == 0:
                        # snap to left edge
                        self.move(QPoint(0, 0))
                        self.resize(QSize(screen.width() / 2, screen.height()))
                        self.setWindowState(Qt.WindowNoState)

                    elif event.globalX() + 1 >= screen.width():
                        # snap to right edge
                        self.move(QPoint(screen.width() / 2, 0))
                        self.resize(QSize(screen.width() / 2, screen.height()))
                        self.setWindowState(Qt.WindowNoState)

                    elif self.geometry().y() < 0:
                        # move window if top of window is outside display
                        self.move(QPoint(self.geometry().x(), 0))

                elif event.type() == QMouseEvent.MouseButtonDblClick:
                    # maximize/restore on double click
                    if self.windowState() == Qt.WindowMaximized:
                        self.setWindowState(Qt.WindowNoState)
                    elif self.windowState() == Qt.WindowNoState:
                        self.showMaximized()

            elif event.type() == QEvent.Leave and widget is self.titlebar():
                # fixes __moving remain True after clicking titlebar on border and leaving titlebar
                self.__moving = False

            elif (self.windowState() not in (Qt.WindowFullScreen, Qt.WindowMaximized)
                    and self.isResizingEnabled()):
                # when cursor is not over titlebar
                # and window is not maximized or in full screen mode

                # button pressed
                if event.type() == QMouseEvent.MouseButtonPress:
                    if self.__griprect.contains(
                            event.pos()) and not self.__horizontalResizing and not self.__verticalResizing:
                        # corner grip
                        self.__resizing = True

                    elif self.__hgriprect.contains(
                            event.pos()) and not self.__resizing and not self.__verticalResizing:
                        # right grip
                        self.__horizontalResizing = True

                    elif self.__vgriprect.contains(
                            event.pos()) and not self.__resizing and not self.__horizontalResizing:
                        # bottom grip
                        self.__verticalResizing = True

                if event.type() == QMouseEvent.MouseButtonRelease:
                    # stop resizing
                    if self.__resizing or self.__horizontalResizing or self.__verticalResizing:
                        self.__resizing = False
                        self.__horizontalResizing = False
                        self.__verticalResizing = False
                        self.setCursor(Qt.ArrowCursor)

                if event.type() == QMouseEvent.MouseMove:
                    # moving cursor while holding left button -> resize

                    if self.__resizing and not self.__horizontalResizing and not self.__verticalResizing:
                        if event.buttons() == Qt.LeftButton:
                            self.setCursor(Qt.SizeFDiagCursor)
                            self.resize(QSize(event.x(), event.y()))

                    elif self.__horizontalResizing and not self.__resizing and not self.__verticalResizing:
                        if event.buttons() == Qt.LeftButton:
                            self.setCursor(Qt.SizeHorCursor)
                            self.resize(QSize(event.x(), self.height()))

                    elif self.__verticalResizing and not self.__resizing and not self.__horizontalResizing:
                        if event.buttons() == Qt.LeftButton:
                            self.setCursor(Qt.SizeVerCursor)
                            self.resize(QSize(self.width(), event.y()))

                    else:
                        if (self.__griprect.contains(event.pos())
                                and not self.__horizontalResizing
                                and not self.__verticalResizing):
                            self.setCursor(Qt.SizeFDiagCursor)
                        elif (self.__hgriprect.contains(event.pos())
                              and not self.__resizing
                              and not self.__verticalResizing):
                            self.setCursor(Qt.SizeHorCursor)
                        elif (self.__vgriprect.contains(event.pos())
                              and not self.__resizing
                              and not self.__horizontalResizing):
                            self.setCursor(Qt.SizeVerCursor)
                        else:
                            self.setCursor(Qt.ArrowCursor)

        return super().eventFilter(widget, event)



"""
The MIT License (MIT)

Copyright (c) 2012-2014 Alexander Turkin
Copyright (c) 2014 William Hallatt
Copyright (c) 2015 Jacob Dawid
Copyright (c) 2016 Luca Weiss
Copyright (c) 2017 fbjorn
Copyright (c) 2020 Wojciech Wentland

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

import math

from qtpy.QtCore import Qt, QTimer, QRect, Signal, QEvent
from qtpy.QtGui import QColor, QPainter
from qtpy.QtWidgets import QWidget

import qrainbowstyle


class WaitingSpinner(QWidget):
    """QtWaitingSpinner is a highly configurable, custom Qt widget
    for showing "waiting" or "loading" spinner icons in Qt applications,
    e.g. the spinners below are all QtWaitingSpinner widgets
    differing only in their configuration

    Args:
        parent (QWidget): Parent widget.
        centerOnParent (bool): Center on parent widget.
        disableParentWhenSpinning (bool): Disable parent widget when spinning.
        modality (Qt.WindowModality): Spinner modality.
        roundness (float): Lines roundness.
        fade (float): Spinner fade.
        lines (int): Lines count.
        line_length (int): Lines length.
        line_width (int): Lines width.
        radius (int): Spinner radius.
        speed (float): Spinner speed.
    """

    def __init__(self, parent, centerOnParent=True, disableParentWhenSpinning=False,
                 modality=Qt.NonModal, roundness=100., fade=80., lines=20,
                 line_length=10, line_width=2, radius=10, speed=math.pi / 2):
        super().__init__(parent)

        self._centerOnParent = centerOnParent
        self._disableParentWhenSpinning = disableParentWhenSpinning

        self._color = QColor(0, 0, 0)
        self._roundness = roundness
        self._minimumTrailOpacity = math.pi
        self._trailFadePercentage = fade
        self._oldTrailFadePercentage = fade
        self._revolutionsPerSecond = speed
        self._numberOfLines = lines
        self._lineLength = line_length
        self._lineWidth = line_width
        self._innerRadius = radius
        self._currentCounter = 0

        self._isSpinning = False

        self.fadeInTimer = QTimer()
        self.fadeOutTimer = QTimer()

        self._timer = QTimer(self)
        self._timer.timeout.connect(self.rotate)
        self.updateSize()
        self.updateTimer()
        self.hide()

        self.setWindowModality(modality)
        self.setAttribute(Qt.WA_TranslucentBackground)
        self.setColor(qrainbowstyle.getCurrentPalette().COLOR_ACCENT_4)

    def paintEvent(self, QPaintEvent):
        self.updatePosition()
        painter = QPainter(self)
        painter.fillRect(self.rect(), Qt.transparent)
        painter.setRenderHint(QPainter.Antialiasing, True)

        if self._currentCounter >= self._numberOfLines:
            self._currentCounter = 0

        painter.setPen(Qt.NoPen)
        for i in range(self._numberOfLines):
            painter.save()
            painter.translate(self._innerRadius + self._lineLength, self._innerRadius + self._lineLength)
            rotateAngle = float(360 * i) / float(self._numberOfLines)
            painter.rotate(rotateAngle)
            painter.translate(self._innerRadius, 0)
            distance = self.lineCountDistanceFromPrimary(i, self._currentCounter, self._numberOfLines)
            color = self.currentLineColor(
                distance,
                self._numberOfLines,
                self._trailFadePercentage,
                self._minimumTrailOpacity,
                self._color
            )
            painter.setBrush(color)
            painter.drawRoundedRect(
                QRect(0, - self._lineWidth / 2, self._lineLength, self._lineWidth),
                self._roundness,
                self._roundness,
                Qt.RelativeSize
            )
            painter.restore()

    def changeEvent(self, event: QEvent):
        """Change event handler.

        Args:
            event (QEvent): Event.
        """
        if event.type() == QEvent.StyleChange:
            self.setColor(qrainbowstyle.getCurrentPalette().COLOR_ACCENT_4)

    def start(self):
        self.updatePosition()
        self._isSpinning = True
        self.show()

        if self.parentWidget and self._disableParentWhenSpinning:
            self.parentWidget().setEnabled(False)

        if not self._timer.isActive():
            self._timer.start()
            self._currentCounter = 0

    def stop(self):
        self._isSpinning = False
        self.hide()

        if self.parentWidget() and self._disableParentWhenSpinning:
            self.parentWidget().setEnabled(True)

        if self._timer.isActive():
            self._timer.stop()
            self._currentCounter = 0

    def setNumberOfLines(self, lines):
        self._numberOfLines = lines
        self._currentCounter = 0
        self.updateTimer()

    def setLineLength(self, length):
        self._lineLength = length
        self.updateSize()

    def setLineWidth(self, width):
        self._lineWidth = width
        self.updateSize()

    def setInnerRadius(self, radius):
        self._innerRadius = radius
        self.updateSize()

    def fadeIn(self, time: int = 15):
        self.setTrailFadePercentage(0)
        self.stopFade()
        self.hide()

        self.fadeInTimer = QTimer()
        self.fadeInTimer.timeout.connect(self._on_fadeIn)
        self.fadeInTimer.start(time)

    def _on_fadeIn(self):
        if self.trailFadePercentage < self._oldTrailFadePercentage:
            if self.trailFadePercentage == 0:
                self.show()
            self.setTrailFadePercentage(self.trailFadePercentage + 1)
        else:
            self.fadeInTimer.stop()

    def fadeOut(self, time: int = 15):
        self.show()
        self.stopFade()

        self.fadeOutTimer = QTimer()
        self.fadeOutTimer.timeout.connect(self._on_fadeOut)
        self.fadeOutTimer.start(time)

    def _on_fadeOut(self):
        if self.trailFadePercentage > 0:
            self.setTrailFadePercentage(self.trailFadePercentage - 1)
        else:
            self.hide()
            self.fadeOutTimer.stop()
            self._isFading = False

    def stopFade(self):
        if self.fadeInTimer.isActive():
            self.fadeInTimer.stop()
        if self.fadeOutTimer.isActive():
            self.fadeOutTimer.stop()

    @property
    def color(self):
        return self._color

    @property
    def roundness(self):
        return self._roundness

    @property
    def minimumTrailOpacity(self):
        return self._minimumTrailOpacity

    @property
    def trailFadePercentage(self):
        return self._trailFadePercentage

    @property
    def revolutionsPersSecond(self):
        return self._revolutionsPerSecond

    @property
    def numberOfLines(self):
        return self._numberOfLines

    @property
    def lineLength(self):
        return self._lineLength

    @property
    def lineWidth(self):
        return self._lineWidth

    @property
    def innerRadius(self):
        return self._innerRadius

    @property
    def isSpinning(self):
        return self._isSpinning

    def setRoundness(self, roundness):
        self._roundness = max(0.0, min(100.0, roundness))

    def setColor(self, color=Qt.black):
        self._color = QColor(color)

    def setRevolutionsPerSecond(self, revolutionsPerSecond):
        self._revolutionsPerSecond = revolutionsPerSecond
        self.updateTimer()

    def setTrailFadePercentage(self, trail):
        self._trailFadePercentage = trail

    def setMinimumTrailOpacity(self, minimumTrailOpacity):
        self._minimumTrailOpacity = minimumTrailOpacity

    def rotate(self):
        self._currentCounter += 1
        if self._currentCounter >= self._numberOfLines:
            self._currentCounter = 0
        self.update()

    def updateSize(self):
        size = (self._innerRadius + self._lineLength) * 2
        self.setFixedSize(size, size)

    def updateTimer(self):
        self._timer.setInterval(1000 / (self._numberOfLines * self._revolutionsPerSecond))

    def updatePosition(self):
        if self.parentWidget() and self._centerOnParent:
            self.move(
                self.parentWidget().width() / 2 - self.width() / 2,
                self.parentWidget().height() / 2 - self.height() / 2
            )

    def lineCountDistanceFromPrimary(self, current, primary, totalNrOfLines):
        distance = primary - current
        if distance < 0:
            distance += totalNrOfLines
        return distance

    def currentLineColor(self, countDistance, totalNrOfLines, trailFadePerc, minOpacity, colorinput):
        color = QColor(colorinput)
        if countDistance == 0:
            return color
        minAlphaF = minOpacity / 100.0
        distanceThreshold = int(math.ceil((totalNrOfLines - 1) * trailFadePerc / 100.0))
        if countDistance > distanceThreshold:
            color.setAlphaF(minAlphaF)
        else:
            alphaDiff = color.alphaF() - minAlphaF
            gradient = alphaDiff / float(distanceThreshold + 1)
            resultAlpha = color.alphaF() - gradient * countDistance
            # If alpha is out of bounds, clip it.
            resultAlpha = min(1.0, max(0.0, resultAlpha))
            color.setAlphaF(resultAlpha)
        return color



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Utilities for processing SASS and images from default and custom palette.
"""

from .__utils import getAllScreensGeometry, getAllScreensWorkspace, getWorkspace, setStylesheetOnQApp, StyleLooper



import sys

if sys.platform == "win32":
    try:
        from PyQt5.QtWinExtras import QtWin
        import win32con
        import win32gui
        import win32api
        import ctypes.wintypes
        from ctypes.wintypes import POINT
    except Exception as e:
        raise ImportError("Could not import required library for Windows support: ".format(e))

else:
    raise Exception("Windows API is not supported on non Windows OS.")

from qtpy.QtWidgets import QApplication
from qtpy.QtCore import QMetaObject, Slot

from .base import FramelessWindowBase


class MINMAXINFO(ctypes.Structure):
    _fields_ = [
        ("ptReserved", POINT),
        ("ptMaxSize", POINT),
        ("ptMaxPosition", POINT),
        ("ptMinTrackSize", POINT),
        ("ptMaxTrackSize", POINT),
    ]


class FramelessWindow(FramelessWindowBase):
    """Frameless window for Windows OS.

    Available features:
        - window moving
        - window resizing
        - maximize on double click on titlebar
        - native snap to borders
        - Aero shake
        - window shadow
    """

    def __init__(self, parent=None):
        super(FramelessWindow, self).__init__(parent)
        self.__rect = QApplication.instance().desktop().availableGeometry(self)

        self.__titlebarHeight = 45
        self.__borderWidth = 3

        self.hwnd = None

        if QtWin.isCompositionEnabled():
            QtWin.extendFrameIntoClientArea(self, -1, -1, -1, -1)
        else:
            QtWin.resetExtendedFrame(self)
        QMetaObject.connectSlotsByName(self)

    def setWindowFlags(self, flags):
        """Sets window flags. Window must be
        re-shown after calling this method.

        Args:
            flags (WindowFlags): Window flags.
        """
        self.hwnd = None
        super().setWindowFlags(flags)
        self.show()

    def show(self):
        """Shows the window."""
        if not self.hwnd:
            self.__setStyle()
        super().show()

    def showNormal(self):
        """Shows the window as normal, i.e. neither maximized, minimized, nor fullscreen."""
        if not self.hwnd:
            self.__setStyle()
        super().showNormal()

    def showMaximized(self):
        """Shows the window as maximized."""
        if not self.hwnd:
            self.__setStyle()
        super().showMaximized()

    def showMinimized(self):
        """Shows the window as minimized."""
        if not self.hwnd:
            self.__setStyle()
        super().showMinimized()

    def showFullScreen(self):
        """Shows the window as fullscreen."""
        if not self.hwnd:
            self.__setStyle()
        super().showFullScreen()

    def showWindowShadow(self, value: bool):
        """Show or hide window shadow.

        Args:
            value (bool): Enable or disable window shadow
        """
        if value:
            QtWin.extendFrameIntoClientArea(self, -1, -1, -1, -1)
        else:
            QtWin.extendFrameIntoClientArea(self, 0, 0, 0, 0)

    def setResizingEnabled(self, value: bool):
        """Enable window resizing

        Args:
            value (bool): Enable or disable window resizing
        """
        self.__resizingEnabled = value

    def setEdgeSnapping(self, value: bool):
        """Enable or disable edge snapping for window.

        Args:
            value (bool): Enable or disable edge snapping.
        """
        if not self.hwnd:
            self.__setStyle()

        if value:
            style = win32gui.GetWindowLong(self.hwnd, win32con.GWL_STYLE)
            win32gui.SetWindowLong(self.hwnd, win32con.GWL_STYLE, style | win32con.WS_TILEDWINDOW)
        else:
            style = win32gui.GetWindowLong(self.hwnd, win32con.GWL_STYLE)
            win32gui.SetWindowLong(
                self.hwnd, win32con.GWL_STYLE, style & ~win32con.WS_OVERLAPPEDWINDOW | win32con.WS_POPUPWINDOW)

    def nativeEvent(self, eventType, message):
        """Handle frameless window native events.

        Args:
            eventType (QByteArray): Event type.
            message (int): Message.
        """
        retval, result = super().nativeEvent(eventType, message)
        if eventType == "windows_generic_MSG":
            msg = ctypes.wintypes.MSG.from_address(int(message))

            if msg.message == win32con.WM_NCCALCSIZE:
                return True, 0

            elif msg.message == win32con.WM_GETMINMAXINFO:
                info = ctypes.cast(
                    msg.lParam, ctypes.POINTER(MINMAXINFO)).contents
                info.ptMaxSize.x = self.__rect.width()
                info.ptMaxSize.y = self.__rect.height() - 1
                info.ptMaxPosition.x, info.ptMaxPosition.y = 0, 0
                return True, 0

            elif msg.message == win32con.WM_NCHITTEST:
                x = win32api.LOWORD(msg.lParam) - self.frameGeometry().x()
                y = win32api.HIWORD(msg.lParam) - self.frameGeometry().y()
                w, h = self.width(), self.height()

                lx = x < self.__borderWidth
                rx = x > w - self.__borderWidth
                ty = y < self.__borderWidth
                by = y > h - self.__borderWidth

                titleBarX = not lx and not rx
                titleBarY = not ty and y < self.__borderWidth + self.titlebar().height()

                if titleBarX and titleBarY:
                    if self.titlebar().mouseOverTitlebar(x, y):
                        return True, win32con.HTCAPTION
                    else:
                        return retval, result

                if not lx and not ty and not rx and not by:
                    return retval, result

                if self.isResizingEnabled():
                    if lx and ty:
                        return True, win32con.HTTOPLEFT
                    if rx and by:
                        return True, win32con.HTBOTTOMRIGHT
                    if rx and ty:
                        return True, win32con.HTTOPRIGHT
                    if lx and by:
                        return True, win32con.HTBOTTOMLEFT
                    if ty:
                        return True, win32con.HTTOP
                    if by:
                        return True, win32con.HTBOTTOM
                    if lx:
                        return True, win32con.HTLEFT
                    if rx:
                        return True, win32con.HTRIGHT

        return retval, result

    def __setStyle(self):
        self.hwnd = int(self.winId())
        style = win32gui.GetWindowLong(self.hwnd, win32con.GWL_STYLE)
        win32gui.SetWindowLong(self.hwnd, win32con.GWL_STYLE, style | win32con.WS_TILEDWINDOW)



html = """
<!DOCTYPE html>
<html>
<head>
    <script src="qrc:///qtwebchannel/qwebchannel.js"></script>
    <meta name="viewport" content="initial-scale=1.0, user-scalable=no">
    <meta charset="utf-8">
    <style>
        #map {
            height: 100%;
        }

        html, body {
            height: 100%;
            margin: 0;
            padding: 0;
        }
    </style>
</head>
<body>
<div id="map"></div>

<script async>

    var jshelper;
    new QWebChannel(qt.webChannelTransport,
                function(channel) { jshelper = channel.objects.jshelper; });
    let map;
    let markers = {};
    let polylines = {};
    let allow_marker_dragging = true;
    let map_draggable = true;
    let map_zoomControl = false;
    let map_scrollwheel = true;
    let map_disableDoubleClickZoom = false;

    window.addEventListener('load', (event) => {
        jshelper.pageIsLoaded();
    });

    window.addEventListener('resize', (event) => {
        jshelper.pageIsResized();
    });

    const image = "https://developers.google.com/maps/documentation/javascript/examples/full/images/beachflag.png";
    var customStyle = [
        {
            "featureType": "water",
            "elementType": "geometry",
            "stylers": [
                {
                    "color": "COLOR_BACKGROUND_1"
                }
            ]
        },
        {
            "featureType": "landscape",
            "elementType": "geometry",
            "stylers": [
                {
                    "color": "COLOR_BACKGROUND_5"
                }
            ]
        },
        {
            "featureType": "road",
            "elementType": "geometry",
            "stylers": [
                {
                    "color": "COLOR_BACKGROUND_3"
                },
                {
                    "lightness": -37
                }
            ]
        },
        {
            "featureType": "poi",
            "elementType": "geometry",
            "stylers": [
                {
                    "color": "COLOR_BACKGROUND_3"
                }
            ]
        },
        {
            "featureType": "transit",
            "elementType": "geometry",
            "stylers": [
                {
                    "color": "COLOR_BACKGROUND_3"
                }
            ]
        },
        {
            "elementType": "labels.text.stroke",
            "stylers": [
                {
                    "visibility": "off"
                },
                {
                    "color": "COLOR_ACCENT_4"
                },
                {
                    "weight": 2
                },
                {
                    "gamma": 0.84
                }
            ]
        },
        {
            "elementType": "labels.text.fill",
            "stylers": [
                {
                    "color": "COLOR_TEXT_1"
                }
            ]
        },
        {
            "elementType": "labels.icon",
            "stylers": [
                {
                    "visibility": "off"
                }
            ]
        },
    ]

    function initMap() {
        map = new google.maps.Map(document.getElementById('map'), {
            zoom: 0,
            center: {lat: 50, lng: 0},
            disableDefaultUI: true,
            mapTypeId: google.maps.MapTypeId.ROADMAP,
            styles: customStyle,
            restriction: {
                latLngBounds: {
                    north: 85.0,
                    south: -85.0,
                    west: -179.9999,
                    east: 179.9999
                },
                strictBounds: true
            }
        });
        map.setOptions({minZoom: 0, maxZoom: 50, draggable: map_draggable, zoomControl: map_zoomControl, scrollwheel: map_scrollwheel, disableDoubleClickZoom: map_disableDoubleClickZoom});

        google.maps.event.addListener(map, "rightclick", function (event) {
            jshelper.mapIsRightClicked(event.latLng.lat(), event.latLng.lng())
        });

        google.maps.event.addListener(map, 'dragend', function () {
            var center = map.getCenter();
            jshelper.mapIsMoved(center.lat(), center.lng());
        });

        google.maps.event.addListener(map, 'click', function (event) {
            jshelper.mapIsClicked(event.latLng.lat(), event.latLng.lng());
        });

        google.maps.event.addListener(map, 'dblclick', function (event) {
            jshelper.mapIsDoubleClicked(event.latLng.lat(), event.latLng.lng());
        });

        google.maps.event.addListenerOnce(map, 'idle', function (){
            jshelper.mapIsFullyLoaded();
        });

        google.maps.event.addListener(map, 'tilesloaded', function () {
            jshelper.tilesAreFullyLoaded();
        });

        console.log("Setup compeleted!");
    }


function addMarker(marker_id, latitude, longitude, params) {
    console.log("Creating new marker: " + marker_id)
    const location = new google.maps.LatLng(latitude, longitude);

    if (marker_id in markers) {
        return updateMarker(marker_id, {position:location})
    }

    const marker = new google.maps.Marker(Object.assign({},
        {position: location,
        map: map,
        draggable: allow_marker_dragging,
        id: marker_id,
        polylines: []},
        params));

    google.maps.event.addListener(marker, 'click', function () {
        jshelper.markerIsClicked(marker_id, marker.position.lat(), marker.position.lng())
    });
    google.maps.event.addListener(marker, 'dblclick', function () {
        jshelper.markerIsDoubleClicked(marker_id, marker.position.lat(), marker.position.lng())
    });
    google.maps.event.addListener(marker, 'rightclick', function () {
         jshelper.markerIsRightClicked(marker_id, marker.position.lat(), marker.position.lng())
    });
    google.maps.event.addListener(marker, 'dragend', function () {
         jshelper.markerIsMoved(marker_id, marker.position.lat(), marker.position.lng())
    });
    markers[marker_id] = marker;
}

function addPolyline(polyline_id, coordsArray) {
    console.log("Creating new polyline: " + polyline_id)
    for (var i = 0; i < coordsArray.length; i++) {
        coordsArray[i]["lat"] = parseFloat(coordsArray[i]["lat"]);
        coordsArray[i]["lng"] = parseFloat(coordsArray[i]["lng"]);
    }

    const connection = new google.maps.Polyline({
        path: coordsArray,
        geodesic: true,
        strokeColor: "#FF0000",
        strokeOpacity: 1.0,
        strokeWeight: 2,
        map: map,
        markers: [],
    });

    google.maps.event.addListener(connection, 'click', function () {
        jshelper.polylineIsClicked(polyline_id, connection.getPath().getArray())
    });
    google.maps.event.addListener(connection, 'dblclick', function () {
        jshelper.polylineIsDoubleClicked(polyline_id, connection.getPath().getArray())
    });
    google.maps.event.addListener(connection, 'rightclick', function () {
         jshelper.polylineIsRightClicked(polyline_id, connection.getPath().getArray())
    });
    polylines[polyline_id] = connection;
}

function addPolylineBetweenMarkers(polyline_id, markersArray) {
    let cordsArray = [];
    for (let marker_id in markersArray) {
        cordsArray.push(markers[marker_id].getPosition());
    }
    addPolyline(polyline_id, cordsArray)
}

function deletePolyline(polyline_id) {
    polylines[polyline_id].setMap(null);
    delete polylines[polyline_id];
}

function getMarkers() {
    let resp = {};
    for (var mid in markers) {
        resp[mid] = {lat: markers[mid].position.lat(), lng: markers[mid].position.lng()};
    }
    return resp;
}

function moveMarker(marker_id, latitude, longitude) {
    let cords = new google.maps.LatLng(latitude, longitude);
    markers[marker_id].setPosition(cords);
}

function deleteMarker(marker_id) {
    markers[marker_id].setMap(null);
    delete markers[marker_id];
}

function updateMarker(marker_id, extras) {
    if (!(marker_id in markers)) {
        return;
    }
    markers[marker_id].setOptions(extras);
}

function enableMarkersDragging(value) {
    allow_marker_dragging = value;
    for (var marker_id in markers) {
        updateMarker(marker_id, {draggable: value});
    }
}

function disableMapDragging(value) {
    map_draggable = value;
    map.setOptions({draggable: value});
}

function showZoomControl(value) {
    map_zoomControl = value;
    map.setOptions({zoomControl: value});
}

function disableScrollWheel(value) {
    map_scrollwheel = value;
    map.setOptions({scrollwheel: value});
}

function disableDoubleClickToZoom(value) {
    map_disableDoubleClickZoom = value;
    map.setOptions({disableDoubleClickZoom: value});
}

function panToCenter() {
    map.panTo(map.getCenter());
}

</script>
<script async
        src="https://maps.googleapis.com/maps/api/js?key=API_KEY_GOES_HERE&callback=initMap">
</script>
</body>
</html>
"""



from qtpy.QtWidgets import QApplication, QDesktopWidget
from qtpy.QtCore import QRect

import qrainbowstyle


def getAllScreensWorkspace() -> list:
    """Returns workspaces of all the available screen.s"""
    return [QDesktopWidget().availableGeometry(x) for x in range(QDesktopWidget().screenCount())]


def getAllScreensGeometry() -> list:
    """Returns geometries of all the available screens."""
    return [QDesktopWidget().screenGeometry(x) for x in range(QDesktopWidget().screenCount())]


def getWorkspace() -> QRect:
    """Returns workspace area of current screen."""
    return QApplication.desktop().availableGeometry()


def setStylesheetOnQApp(style):
    """Set stylesheet on current app."""
    app = QApplication.instance()
    app.setStyleSheet(qrainbowstyle.load_stylesheet(style=style))


class StyleLooper:
    def __init__(self):
        super(StyleLooper, self).__init__()

        self.style_index = 0
        self.styles = qrainbowstyle.getAvailableStyles()

    def change(self):
        self.style_index = self.style_index + 1
        if self.style_index >= len(self.styles):
            self.style_index = 0

        setStylesheetOnQApp(self.styles[self.style_index])



import os
import sys


if sys.platform == "win32" and not os.getenv("FORCE_NON_WIN"):
    from .FramelessWindowWin import FramelessWindow
    import logging
    logging.getLogger().info("Imported frameless windows with Windows API support")
else:
    from .FramelessWindow import FramelessWindow

from .FramelessMessageBox import (FramelessWarningMessageBox, FramelessQuestionMessageBox,
                                  FramelessInformationMessageBox, FramelessCriticalMessageBox, FramelessMessageBox)  # noqa



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""MapsView allows to load Google Maps to Qt app.

Usage

.. code-block:: python

    import qrainbowstyle
    import qrainbowstyle.widgets

Work in progress...
"""
import json

from qtpy.QtCore import QObject, Slot, QEvent, Signal, Qt, QUrl
from qtpy.QtWebEngineWidgets import QWebEngineView, QWebEngineSettings, QWebEnginePage
from qtpy.QtNetwork import QNetworkProxyFactory
from qtpy.QtWebChannel import QWebChannel

import qrainbowstyle
import logging

from . import GoogleMapsHtml, OpenStreetMapsHtml


def convertBoolean(value: bool):
    """Convert Python bool to JS boolean.

    Args:
        value (bool): True/False
    """
    if value:
        jvalue = "true"
    else:
        jvalue = "false"
    return jvalue


class CallHandler(QObject):
    """Google Maps events handler. Emits signals for
    map, markers, polylines - clicked, doubleClicked, rightClicked,
    moved events.

    Args:
        parent (object): Parent object.
    """
    runJavascript = Signal(str, object)

    mapMoved = Signal(float, float)
    mapClicked = Signal(float, float)
    mapRightClicked = Signal(float, float)
    mapDoubleClicked = Signal(float, float)

    markerMoved = Signal(str, float, float)
    markerClicked = Signal(str, float, float)
    markerDoubleClicked = Signal(str, float, float)
    markerRightClicked = Signal(str, float, float)

    polylineClicked = Signal(str, list)
    polylineRightClicked = Signal(str, list)
    polylineDoubleClicked = Signal(str, list)

    pageLoaded = Signal()
    pageResized = Signal()
    mapLoaded = Signal()
    tilesLoaded = Signal()

    def __init__(self, parent=None):
        super(CallHandler, self).__init__(parent)
        self._logger = logging.getLogger(self.__class__.__name__)
        self.markers = []

        self._loaded = False
        self._exec_later = []

    def runScript(self, script, callback=None):
        """Run Javascript code.

        Args:
            script (str): Script to execute.
            callback (callback, optional): Function to handle callback.
        """
        self.runJavascript.emit(script, callback)

    @Slot(float, float)
    def mapIsRightClicked(self, lat, lng):
        """Handle right clicks on map.

        Args:
            lat (float): Event latitude.
            lng (float): Event longitude.
        """
        self._logger.debug("Right clicked on: " + str(lat) + ":" + str(lng))
        self.mapRightClicked.emit(lat, lng)

    @Slot(float, float)
    def mapIsMoved(self, lat, lng):
        """Handle moving the map event.

        Args:
            lat (float): Event latitude.
            lng (float): Event longitude.
        """
        self._logger.debug("Moved, center: " + str(lat) + ":" + str(lng))
        self.mapMoved.emit(lat, lng)

    @Slot(float, float)
    def mapIsClicked(self, lat, lng):
        """Handle clicks on map.

        Args:
            lat (float): Event latitude.
            lng (float): Event longitude.
        """
        self._logger.debug("Clicked on: " + str(lat) + ":" + str(lng))
        self.mapClicked.emit(lat, lng)

    @Slot(float, float)
    def mapIsDoubleClicked(self, lat, lng):
        """Handle double clicks on map.

        Args:
            lat (float): Event latitude.
            lng (float): Event longitude.
        """
        self._logger.debug("Double clicked on: " + str(lat) + ":" + str(lng))
        self.mapDoubleClicked.emit(lat, lng)

    @Slot(str, float, float)
    def markerIsClicked(self, marker_id, lat, lng):
        """Handle clicks on markers.

        Args:
            marker_id (str): Marker id.
            lat (float): Event latitude.
            lng (float): Event longitude.
        """
        self._logger.debug("Marker " + str(marker_id) + " clicked: " + str(lat) + ":" + str(lng))
        self.markerClicked.emit(marker_id, lat, lng)

    @Slot(str, float, float)
    def markerIsRightClicked(self, marker_id, lat, lng):
        """Handle right clicks on markers.

        Args:
            marker_id (str): Marker id.
            lat (float): Event latitude.
            lng (float): Event longitude.
        """
        self._logger.debug("Marker " + str(marker_id) + " right clicked: " + str(lat) + ":" + str(lng))
        self.markerRightClicked.emit(marker_id, lat, lng)

    @Slot(str, float, float)
    def markerIsDoubleClicked(self, marker_id, lat, lng):
        """Handle double clicks on markers.

        Args:
            marker_id (str): Marker id.
            lat (float): Event latitude.
            lng (float): Event longitude.
        """
        self._logger.debug("Marker " + str(marker_id) + " double clicked: " + str(lat) + ":" + str(lng))
        self.markerDoubleClicked.emit(marker_id, lat, lng)

    @Slot(str, float, float)
    def markerIsMoved(self, marker_id, lat, lng):
        """Handle moving markers.

        Args:
            marker_id (str): Marker id.
            lat (float): Event latitude.
            lng (float): Event longitude.
        """
        self._logger.debug("Marker " + str(marker_id) + " moved to: " + str(lat) + ":" + str(lng))
        self.moveMarker(marker_id, lat, lng)

    @Slot(str, list)
    def polylineIsClicked(self, polyline_id, path: list):
        """Handle clicks on polylines.

        Args:
            polyline_id (str): Polyline id.
            path (list): List of coordinates.
        """
        self._logger.debug("Polyline " + str(polyline_id) + " clicked: " + str(path))
        self.polylineClicked.emit(polyline_id, path)

    @Slot(str, list)
    def polylineIsRightClicked(self, polyline_id, path: list):
        """Handle right clicks on polylines.

        Args:
            polyline_id (str): Polyline id.
            path (list): List of coordinates.
        """
        self._logger.debug("Polyline " + str(polyline_id) + " right clicked: " + str(path))
        self.polylineRightClicked.emit(polyline_id, path)

    @Slot(str, list)
    def polylineIsDoubleClicked(self, polyline_id, path: list):
        """Handle double clicks on polylines.

        Args:
            polyline_id (str): Polyline id.
            path (list): List of coordinates.
        """
        self._logger.debug("Polyline " + str(polyline_id) + " double clicked: " + str(path))
        self.polylineDoubleClicked.emit(polyline_id, path)

    @Slot()
    def pageIsLoaded(self):
        """Triggered when QWebEngineView finishes loading page.
        Emits pageLoaded signal."""
        self.pageLoaded.emit()

    @Slot()
    def pageIsResized(self):
        """Triggered when map widget is resized. Emits pageResized signal."""
        self.pageResized.emit()

    @Slot()
    def mapIsFullyLoaded(self):
        """Triggered when map finishes loading. Emits mapLoaded signal.
        It may be triggered before showing the map."""
        self.mapLoaded.emit()

    @Slot()
    def tilesAreFullyLoaded(self):
        """Triggered when map finish loading tiles. Emits tilesLoaded signal.
        It is last signal emited after creating MapsView widget."""
        self.tilesLoaded.emit()

    def _updateMarkersCallback(self, markers: list):
        """Callback for loading marker list from QWebEngineView."""
        self.markers = markers

    def loadMarkers(self):
        """Updates markers list."""
        self.runScript("getMarkers();", self._updateMarkersCallback)

    def updateMarker(self, marker_id, args):
        """Update markers parameters."""
        return self.runScript("updateMarker({}, {});".format(marker_id, json.dumps(args)))

    def moveMarker(self, marker_id, latitude, longitude):
        """Move marker to provided latitude and longitude."""
        return self.runScript("moveMarker({}, {}, {});".format(marker_id, latitude, longitude))

    def deleteMarker(self, marker_id):
        """Delete marker with provided ID."""
        return self.runScript("deleteMarker({});".format(marker_id))

    def addMarker(self, marker_id, latitude, longitude, options):
        """Creates marker with marker_id id at latitude, longitude.

        Args:
            marker_id (int): Marker id.
            latitude (float): Marker latitude.
            longitude (float): Marker longitude.
            options (dict): Marker options.
        """
        return self.runScript("addMarker({}, {}, {}, {});".format(marker_id, latitude, longitude, json.dumps(options)))

    def addPolyline(self, polyline_id, coordinates: list):
        """Creates polyline between coordinates.

        Args:
            polyline_id (int): Polyline ID.
            coordinates (list): List of coordinates (dicts with "lat" and "lng" keys).
        """
        return self.runScript("addPolyline({}, {});".format(polyline_id, coordinates))

    def addPolylineBetweenMarkers(self, polyline_id, markers_ids: list):
        """Creates polyline between markers.

        Args:
            polyline_id (int): Polyline ID.
            markers_ids (list): List of markers IDs.
        """
        return self.runScript("addPolylineBetweenMarkers({}, {});".format(polyline_id, markers_ids))

    def deletePolyline(self, polyline_id):
        """Delete polyline with provided ID.

        Args:
            polyline_id (int): Polyline ID.
        """
        return self.runScript("deletePolyline({});".format(polyline_id))

    def panToCenter(self):
        """Pan map to center."""
        if self._loaded:
            return self.runScript("panToCenter();")
        else:
            self._exec_later.append(lambda: self.panToCenter())

    def disableMapDragging(self, value):
        """Enable or disable map dragging.

        Args:
            value (bool): Map dragging status.
        """
        if self._loaded:
            return self.runScript("disableMapDragging({});".format(convertBoolean(not value)))
        else:
            self._exec_later.append(lambda: self.disableMapDragging(value))

    def showZoomControl(self, value):
        """Show or hide zoom control widget.

        Args:
            value (bool): Zoom control widget status.
        """
        if self._loaded:
            return self.runScript("showZoomControl({});".format(convertBoolean(value)))
        else:
            self._exec_later.append(lambda: self.showZoomControl(value))

    def disableDoubleClickToZoom(self, value):
        """Enable or disable double click to zoom.

        Args:
            value (bool): Double click to zoom status.
        """
        if self._loaded:
            return self.runScript("disableDoubleClickToZoom({})".format(convertBoolean(value)))
        else:
            self._exec_later.append(lambda: self.disableDoubleClickToZoom(value))

    def disableScrollWheel(self, value):
        """Enable or disable scroll to zoom.

        Args:
            value (bool): Scroll to zoom status.
        """
        if self._loaded:
            return self.runScript("disableScrollWheel({});".format(convertBoolean(not value)))
        else:
            self._exec_later.append(lambda: self.disableScrollWheel(value))

    def enableMarkersDragging(self, value):
        """Enable or disable markers dragging feature.

        Args:
            value (bool): Enable markers dragging.
        """
        if self._loaded:
            return self.runScript("enableMarkersDragging({});".format(convertBoolean(value)))
        else:
            self._exec_later.append(lambda: self.enableMarkersDragging(value))

    def on_loadFinished(self):
        """Set loaded flag to True."""
        self._loaded = True
        for setting in self._exec_later:
            try:
                setting()
            except Exception as e:
                self._logger.warning("Exception while executing setting: {}".format(e))

    def on_loadStarted(self):
        """Set loaded flag to False."""
        self._loaded = False


class MapsPage(QWebEnginePage):
    """QWebEngineView page for handling Javascript console messages."""
    message = Signal(dict)

    def __init__(self, parent=None):
        super(MapsPage, self).__init__(parent)
        self._logger = logging.getLogger(self.__class__.__name__)
        self.setBackgroundColor(Qt.transparent)

    def javaScriptConsoleMessage(self, level, msg, line, source_id):
        """Handle Javascript console messages.

        Args:
            level (int): Logging level.
            msg (str): Message string.
            line (int): Line in code where error occured.
            source_id (str): Element ID.
        """
        self._logger.log(10 * (level + 1), "[{}]: {}".format(line, msg))
        self.message.emit({"level": 10 * (level + 1), "msg": msg, "line": line, "source_id": source_id})


class MapsView(QWebEngineView):
    """Show Google Maps in Qt app."""

    def __init__(self, parent):
        super(MapsView, self).__init__(parent)
        self._logger = logging.getLogger(self.__class__.__name__)

        # Set browser attributes
        QNetworkProxyFactory.setUseSystemConfiguration(False)
        self.settings().setAttribute(QWebEngineSettings.Accelerated2dCanvasEnabled, True)
        self.settings().setAttribute(QWebEngineSettings.JavascriptEnabled, True)

        # Create maps page (it is not needed, but we can handle Javascript console logs)
        self._mapspage = MapsPage(self)
        self.setPage(self._mapspage)

        # Create connection between Javascript and Qt
        self.channel = QWebChannel(self.page())
        self.page().setWebChannel(self.channel)

        # create map events handler and register it as "jshelper" in HTML
        self.handler = CallHandler(self)
        self.handler.runJavascript.connect(self.runScript)
        self.loadFinished.connect(self.handler.on_loadFinished)
        self.loadStarted.connect(self.handler.on_loadStarted)

        self.channel.registerObject("jshelper", self.handler)

    @Slot(str, object)
    def runScript(self, script, callback):
        """Run Javascript code.

        Args:
            script (str): Script to execute.
            callback (callback, optional): Function to handle callback.
        """
        if callback is None:
            self.page().runJavaScript(script)
        else:
            self.page().runJavaScript(script, callback)

    def getHandler(self):
        """Returns map event handler."""
        return self.handler

    def enableMarkersDragging(self, value):
        """Enable or disable markers dragging feature.

        Args:
            value (bool): Enable markers dragging.
        """
        self.handler.enableMarkersDragging(value)

    def addMarker(self, marker_id, lat, lng, options=None):
        """Creates marker with marker_id id at latitude, longitude.

        Args:
            marker_id (int): Marker ID.
            lat (float): Marker latitude.
            lng (float): Marker longitude.
            options (dict): Marker options.
        """
        if options is None:
            options = {}
        self.handler.addMarker(marker_id, lat, lng, options)

    def deleteMarker(self, marker_id):
        """Delete marker with ID.

        Args:
            marker_id (int): Marker ID.
        """
        self.handler.deleteMarker(marker_id)

    def updateMarker(self, marker_id, options):
        """Delete marker with ID.

        Args:
            marker_id (int): Marker ID.
            options (dict): Marker options.
        """
        self.handler.updateMarker(marker_id, options)

    def moveMarker(self, marker_id, lat, lng):
        """Move marker to location with (lat, lng).

        Args:
            marker_id (int): Marker ID.
            lat (float): Marker latitude.
            lng (float): Marker longitude.
        """
        self.handler.moveMarker(marker_id, lat, lng)

    def deletePolyline(self, polyline_id):
        """Delete polyline with ID.

        Args:
            polyline_id (int): Polyline ID.
        """
        self.handler.deletePolyline(polyline_id)

    def addPolyline(self, polyline_id, coords: list):
        """Creates polyline using coordinates.

        Args:
            polyline_id (int): Polyline id
            coords (list): List of coordinates (dicts with "lat" and "lng" keys).
        """
        self.handler.addPolyline(polyline_id, coords)

    def addPolylineBetweenMarkers(self, polyline_id, markers: list):
        """Creates polyline using coordinates.

        Args:
            polyline_id (int): Polyline id
            markers (list): List of markers IDs.
        """
        self.handler.addPolylineBetweenMarkers(polyline_id, markers)

    def panToCenter(self):
        """Pan map to center."""
        self.handler.panToCenter()

    def disableMapDragging(self, value):
        """Enable or disable map dragging.

        Args:
            value (bool): Map dragging status.
        """
        self.handler.disableMapDragging(value)

    def showZoomControl(self, value):
        """Show or hide zoom control widget.

        Args:
            value (bool): Zoom control widget status.
        """
        self.handler.showZoomControl(value)

    def disableDoubleClickToZoom(self, value):
        """Enable or disable double click to zoom.

        Args:
            value (bool): Double click to zoom status.
        """
        self.handler.disableDoubleClickToZoom(value)

    def disableScrollWheel(self, value):
        """Enable or disable scroll to zoom.

        Args:
            value (bool): Scroll to zoom status.
        """
        self.handler.disableScrollWheel(value)


class GoogleMapsView(MapsView):

    def __init__(self, parent, api_key):
        super(GoogleMapsView, self).__init__(parent)
        self.api_key = api_key

    def changeEvent(self, event: QEvent):
        """Change event handler.
        Args:
            event (QEvent): Event.
        """
        if event.type() == QEvent.StyleChange:
            self.setHtml(qrainbowstyle.rainbowize(GoogleMapsHtml.html.replace("API_KEY_GOES_HERE", self.api_key)))
        return super().changeEvent(event)


class OpenStreetMapsView(MapsView):

    def __init__(self, parent):
        super(OpenStreetMapsView, self).__init__(parent)

    def changeEvent(self, event: QEvent):
        """Change event handler.
        Args:
            event (QEvent): Event.
        """
        if event.type() == QEvent.StyleChange:
            self.setHtml(qrainbowstyle.rainbowize(OpenStreetMapsHtml.html))
        return super().changeEvent(event)



from qtpy.QtWidgets import QGridLayout, QHBoxLayout, QVBoxLayout, QWidget, QPushButton, QApplication
from qtpy.QtCore import Signal, QSize
from qtpy.QtGui import QPixmap, QPainter, QPen, QColor, QIcon

import qrainbowstyle
from qrainbowstyle.utils import setStylesheetOnQApp


class Painter(QWidget):
    """Painter documentation"""

    def __init__(self, parent=None):
        super(Painter, self).__init__(parent)
        self.pixmap = None
        self.c1 = None
        self.c2 = None
        self.size = None

    def paintPixmap(self, size, c1, c2):
        self.size = size
        self.c1 = c1
        self.c2 = c2
        self.pixmap = QPixmap(size, size)

        self.paintEvent(None)
        return self.pixmap

    def paintEvent(self, event):
        painter = QPainter()
        painter.begin(self.pixmap)
        painter.drawPixmap(0, 0, self.pixmap)

        pen1 = QPen(QColor(self.c1), 1)
        pen2 = QPen(QColor(self.c2), 2)

        painter.setPen(pen1)
        y = self.size - 1
        for x in range(self.size):
            painter.drawLine(0, x, y, x)
            y = y - 1

        painter.setPen(pen2)
        y = 1
        for x in range(self.size, 0, -1):
            painter.drawLine(y, x, self.size, x)
            y = y + 1

        painter.end()


class _PaletteButton(QPushButton):
    def __init__(self, color):
        super().__init__()
        side = 128
        self.setFixedSize(QSize(side, side))
        self.color = color

        self.setIcon(QIcon(Painter().paintPixmap(side, self.color[1], self.color[2])))
        stylesheet = """
        min-width: 24px;
        max-width: 24px;
        min-height: 24px;
        max-height: 24px;
        padding: 0px;
        border: none;
        """
        self.setIconSize(QSize(24, 24))
        self.setStyleSheet(stylesheet)


class _PaletteBase(QWidget):
    selected = Signal(object)

    def _emit_color(self, color):
        self.selected.emit(color)


class _PaletteLinearBase(_PaletteBase):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        colors = []

        for style in qrainbowstyle.getAvailablePalettes():
            colors.append({1: style.COLOR_BACKGROUND_1, 2: style.COLOR_ACCENT_3, "name": style.__name__})

        palette = self.layoutvh()

        for c in colors:
            b = _PaletteButton(c)
            b.pressed.connect(lambda color=c: setStylesheetOnQApp(style=color["name"]))
            palette.addWidget(b)

        self.setLayout(palette)


class StylePickerHorizontal(_PaletteLinearBase):
    layoutvh = QHBoxLayout


class StylePickerVertical(_PaletteLinearBase):
    layoutvh = QVBoxLayout


class StylePickerGrid(QWidget):
    """Select application color palette from a grid."""

    def __init__(self, n_columns=5, parent=None):
        super(StylePickerGrid, self).__init__(parent)
        self.setMaximumWidth(150)
        colors = []

        for style in qrainbowstyle.getAvailablePalettes():
            colors.append({1: style.COLOR_BACKGROUND_1, 2: style.COLOR_ACCENT_3, "name": style.__name__})

        palette = QGridLayout()
        row, col = 0, 0

        for c in colors:
            b = _PaletteButton(c)
            b.pressed.connect(lambda color=c: setStylesheetOnQApp(style=color["name"]))
            palette.addWidget(b, row, col)
            col += 1
            if col == n_columns:
                col = 0
                row += 1

        self.setLayout(palette)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
A customizable style sheet for QtWidgets application.
"""

# Standard library imports
import os
import sys
import glob
from setuptools import find_packages, setup

# Package imports
from qrainbowstyle import __doc__ as long_desc
from qrainbowstyle import __version__

install_requires = ['helpdev>=0.6.10', 'qtpy>=1.9', 'PyQtWebEngine']
if sys.platform.startswith("win"):
    install_requires += ["pywin32"]

extras_require = {
    'develop': ['qtsass', 'watchdog'],
    'docs': ['sphinx', 'sphinx_rtd_theme'],
    'example': ['pyqt5', 'pyside2']
}


def remove_all(dir_path, patterns='*.pyc'):
    """Remove all files from `dir_path` matching the `patterns`.

    Args:
        dir_path (str): Directory path.
        patterns (str): Pattern using regex. Defaults to '*.pyc'.
    """

    for pattern in patterns:
        for filename in glob.iglob(dir_path + '/**/' + pattern, recursive=True):
            os.remove(filename)


setup(
    name='qrainbowstyle',
    version=__version__,
    packages=find_packages(),
    url='https://github.com/desty2k/QRainbowStyleSheet',
    license='MIT',
    author='Wojciech Wentland',
    author_email='wojciech.wentland@int.pl',
    description='The most complete customizable stylesheet for Python and Qt applications',
    long_description=long_desc,
    long_description_content_type='text/x-rst',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: X11 Applications :: Qt',
        'Environment :: Win32 (MS Windows)',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: POSIX :: Linux',
        'Operating System :: MacOS',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Topic :: Software Development :: Libraries :: Application Frameworks'
    ],
    python_requires='>=3.5',
    zip_safe=False,  # don't use eggs
    entry_points={"console_scripts": ["qrainbowstyle=qrainbowstyle.__main__:main"]},
    extras_require=extras_require,
    install_requires=install_requires,
    project_urls={
        "Issues": "https://github.com/desty2k/QRainbowStyleSheet/issues",
        "Docs": "https://desty2k.github.io/QRainbowStyleSheet/",
    }
)



html = """
<!doctype html>
<html lang="en">

  <head>
    <link rel="stylesheet" href="https://cdn.jsdelivr.net/gh/openlayers/openlayers.github.io@master/en/v6.5.0/css/ol.css" type="text/css">
    <style>
        .map {
            height: 100%;
        }

        html, body {
            height: 100%;
            margin: 0;
            padding: 0;
        }
    </style>
    <script src="qrc:///qtwebchannel/qwebchannel.js"></script>
    <script src="https://cdn.jsdelivr.net/gh/openlayers/openlayers.github.io@master/en/v6.5.0/build/ol.js"></script>
    <title>OpenLayers example</title>
  </head>

  <body>
    <div id="map" class="map"></div>

    <script type="text/javascript" async>
        let jshelper;
        let map;
        let view;
        let markers = [];

        new QWebChannel(qt.webChannelTransport,
                function(channel) { jshelper = channel.objects.jshelper; });
        var vectorSource = new ol.source.Vector({
            features: markers
        });
        var vectorLayer = new ol.layer.Vector({
            source: vectorSource,
        });

        function initMap() {
            view = new ol.View({
                center: ol.proj.fromLonLat([0, 45]),
                zoom: 0,
                projection: "EPSG:3857",
                extent: ol.proj.get("EPSG:3857").getExtent(),
            });

            map = new ol.Map({
                target: 'map',
                layers: [
                    new ol.layer.Tile({
                        source: new ol.source.OSM()
                    })
                ],
                view: view,
            });
            map.addLayer(vectorLayer);
        }

        function addMarker(marker_id, longitude, latitude, params) {
            var iconGeometry=new ol.geom.Point(ol.proj.transform([latitude, longitude], 'EPSG:4326', 'EPSG:3857'))
            var iconFeature = new ol.Feature({
                geometry:iconGeometry
            });
            iconFeature.setId(marker_id)
            vectorSource.addFeature(iconFeature);
        }

    </script>

    <script async>
        initMap()
    </script>
  </body>
</html>
"""



from qtpy.QtWidgets import QWidget, QSizePolicy, QHBoxLayout, QToolButton, QLabel, QPushButton
from qtpy.QtGui import QRegion
from qtpy.QtCore import Qt, QSize, QRect

import qrainbowstyle


class MenuButton(QToolButton):
    """MenuButton documentation"""

    def __init__(self, parent):
        super(MenuButton, self).__init__(parent)

        self.setPopupMode(QToolButton.InstantPopup)
        self.setMouseTracking(True)
        self.setSizePolicy(QSizePolicy.Minimum, QSizePolicy.Expanding)


class AppLogo(QLabel):
    """
        Label with app logo.
    """

    def __init__(self, parent):
        super(AppLogo, self).__init__(parent)
        self.setScaledContents(True)
        self.setFixedSize(QSize(28, 28))
        self.setStyleSheet("border: none;")


class TitlebarWindowsButton(QPushButton):

    def __init__(self, parent):
        super(TitlebarWindowsButton, self).__init__(parent)
        iconsize = QSize(45, 30)
        self.setIconSize(iconsize)
        self.setMinimumSize(iconsize)
        self.setText("")
        self.setEnabled(True)
        self.setAutoFillBackground(True)
        self.setChecked(False)
        self.setMouseTracking(True)

        sizepolicy = QSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)
        sizepolicy.setHorizontalStretch(0)
        sizepolicy.setVerticalStretch(0)
        sizepolicy.setHeightForWidth(self.sizePolicy().hasHeightForWidth())
        self.setSizePolicy(sizepolicy)


class MinimizeWindowsButton(TitlebarWindowsButton):

    def __init__(self, parent=None):
        super(MinimizeWindowsButton, self).__init__(parent)


class MaximizeWindowsButton(TitlebarWindowsButton):

    def __init__(self, parent=None):
        super(MaximizeWindowsButton, self).__init__(parent)


class RestoreWindowsButton(TitlebarWindowsButton):

    def __init__(self, parent=None):
        super(RestoreWindowsButton, self).__init__(parent)


class CloseWindowsButton(TitlebarWindowsButton):

    def __init__(self, parent=None):
        super(CloseWindowsButton, self).__init__(parent)


class CloseSquareWindowsButton(TitlebarWindowsButton):

    def __init__(self, parent=None):
        super(CloseSquareWindowsButton, self).__init__(parent)
        iconsize = QSize(30, 30)
        self.setIconSize(iconsize)
        self.setMinimumSize(iconsize)


class TitlebarDarwinButton(QPushButton):

    def __init__(self, parent):
        super(TitlebarDarwinButton, self).__init__(parent)
        self.setIconSize(QSize(15, 15))
        self.setMinimumSize(QSize(15, 15))
        self.setAutoFillBackground(True)
        self.setMouseTracking(True)
        sizepolicy = QSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)
        sizepolicy.setHorizontalStretch(0)
        sizepolicy.setVerticalStretch(0)
        sizepolicy.setHeightForWidth(self.sizePolicy().hasHeightForWidth())
        self.setSizePolicy(sizepolicy)


class MinimizeDarwinButton(TitlebarDarwinButton):

    def __init__(self, parent=None):
        super(MinimizeDarwinButton, self).__init__(parent)


class MaximizeDarwinButton(TitlebarDarwinButton):

    def __init__(self, parent=None):
        super(MaximizeDarwinButton, self).__init__(parent)


class RestoreDarwinButton(TitlebarDarwinButton):

    def __init__(self, parent=None):
        super(RestoreDarwinButton, self).__init__(parent)


class CloseDarwinButton(TitlebarDarwinButton):

    def __init__(self, parent=None):
        super(CloseDarwinButton, self).__init__(parent)


class ButtonsWidget(QWidget):
    """Widget with titlebar buttons"""

    def __init__(self, parent):
        super(ButtonsWidget, self).__init__(parent)
        self.setObjectName("buttonsWidget")
        self.setContentsMargins(0, 0, 0, 0)
        sizepolicy = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Preferred)
        sizepolicy.setHorizontalStretch(0)
        sizepolicy.setVerticalStretch(0)
        sizepolicy.setHeightForWidth(self.sizePolicy().hasHeightForWidth())
        self.setSizePolicy(sizepolicy)
        self.setStyleSheet("padding: 0px;")

        self.buttonsLayout = QHBoxLayout(self)
        self.buttonsLayout.setContentsMargins(0, 0, 0, 0)
        self.buttonsLayout.setSpacing(0)
        self.buttonsLayout.setAlignment(Qt.AlignVCenter)
        self.setLayout(self.buttonsLayout)
        if qrainbowstyle.USE_DARWIN_BUTTONS:
            self.btnMinimize = MinimizeDarwinButton(self)
            self.btnMaximize = MaximizeDarwinButton(self)
            self.btnRestore = RestoreDarwinButton(self)
            self.btnClose = CloseDarwinButton(self)
        else:
            self.btnMinimize = MinimizeWindowsButton(self)
            self.btnMaximize = MaximizeWindowsButton(self)
            self.btnRestore = RestoreWindowsButton(self)
            self.btnClose = CloseWindowsButton(self)

        if qrainbowstyle.ALIGN_BUTTONS_LEFT:
            self.buttonsLayout.addWidget(self.btnClose)
            if not qrainbowstyle.USE_DARWIN_BUTTONS:
                self.buttonsLayout.addWidget(self.btnRestore)
            self.buttonsLayout.addWidget(self.btnMinimize)
            self.buttonsLayout.addWidget(self.btnMaximize)
        else:
            self.buttonsLayout.addWidget(self.btnMinimize)
            self.buttonsLayout.addWidget(self.btnMaximize)
            if not qrainbowstyle.USE_DARWIN_BUTTONS:
                self.buttonsLayout.addWidget(self.btnRestore)
            self.buttonsLayout.addWidget(self.btnClose)

        self.btnMinimize.setObjectName("btnMinimize")
        self.btnMaximize.setObjectName("btnMaximize")
        if not qrainbowstyle.USE_DARWIN_BUTTONS:
            self.btnRestore.setObjectName("btnRestore")
        self.btnClose.setObjectName("btnClose")

        if qrainbowstyle.USE_DARWIN_BUTTONS:
            self.buttonsLayout.setSpacing(8)



from qtpy.QtCore import Qt, QSize
from qtpy.QtWidgets import QGridLayout, QLabel, QStyle, QDialogButtonBox, QSizePolicy, QWidget, QApplication
from qtpy.QtGui import QIcon

from . import FramelessWindow


class FramelessMessageBox(FramelessWindow):
    """Frameless messagebox."""

    def __init__(self, icon=None, parent=None):
        super(FramelessMessageBox, self).__init__(parent)
        self.setResizingEnabled(False)
        self.resize(QSize(350, 150))
        policy = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Minimum)
        policy.setHeightForWidth(True)
        self.setSizePolicy(policy)

        self.__messagewidget = QWidget(self)
        self.__messagewidget.setContentsMargins(11, 11, 11, 11)

        self.__grid = QGridLayout(self.__messagewidget)
        self.__grid.setContentsMargins(0, 0, 0, 0)
        self.__grid.setVerticalSpacing(8)
        self.__grid.setHorizontalSpacing(0)

        self.__messagewidget.setLayout(self.__grid)

        self.__iconLabel = QLabel(self.__messagewidget)
        self.__iconLabel.setSizePolicy(QSizePolicy.Minimum, QSizePolicy.Minimum)
        self.__iconLabel.setScaledContents(True)
        self.__grid.addWidget(self.__iconLabel, 0, 0, 2, 1, Qt.AlignTop | Qt.AlignLeft)

        self.__textLabel = QLabel(self.__messagewidget)
        self.__textLabel.setWordWrap(True)
        self.__textLabel.setContentsMargins(2, 0, 0, 0)
        self.__textLabel.setIndent(9)
        self.__textLabel.setScaledContents(True)
        self.__textLabel.setMouseTracking(True)
        self.__textLabel.autoFillBackground()
        self.__textLabel.setSizePolicy(QSizePolicy.Minimum, QSizePolicy.Minimum)
        self.__grid.addWidget(self.__textLabel, 0, 1, 1, 1)

        self.__grid.setRowStretch(1, 100)
        self.__grid.setRowMinimumHeight(2, 6)

        self.__buttonBox = QDialogButtonBox(self.__messagewidget)
        self.__buttonBox.setStandardButtons(QDialogButtonBox.Ok | QDialogButtonBox.Cancel)
        self.__grid.addWidget(self.__buttonBox, 3, 1, 1, 1)

        self.addContentWidget(self.__messagewidget)
        self.setIcon(icon)

        self.showSizeControl(False)
        self.setEdgeSnapping(False)

    def setStandardButtons(self, buttons):
        """Set standard buttons for message box.

        Args:
            buttons (StandardButtons): Standard buttons.
        """
        self.__buttonBox.setStandardButtons(buttons)

    def addButton(self, button, role):
        """Adds the given button to the button box with the specified role.
        If the role is invalid, the button is not added.

        If the button has already been added, it is removed and added again with the new role.

        Note: The button box takes ownership of the button.

        Args:
            button (StandardButton): Standard button.
            role (ButtonRole): Button role
        """
        self.__buttonBox.addButton(button, role)

    def removeButton(self, button):
        """Removes button from the button box without deleting it and sets its parent to zero.

        Args:
            button (StandardButton): Standard button.
        """
        self.__buttonBox.removeButton(button)

    def button(self, button):
        """Returns the QPushButton corresponding to the standard button which,
        or 0 if the standard button doesn't exist in this button box.

        Args:
            button (StandardButton): Standard button.
        """
        return self.__buttonBox.button(button)

    def buttons(self):
        """Returns a list of all the buttons that have been added to the button box."""
        return self.__buttonBox.buttons()

    def standardButtons(self):
        """This property holds collection of standard buttons in the button box.
        This property controls which standard buttons are used by the button box."""
        return self.__buttonBox.standardButtons()

    def standardButton(self, button):
        """Returns the standard button enum value corresponding to the given button,
        or NoButton if the given button isn't a standard button.

        Args:
            button (QAbstractButton): Standard button.
        """
        return self.__buttonBox.standardButton(button)

    def setText(self, text: str):
        """Set text for message box."""
        self.__textLabel.setText(text)
        self.__textLabel.adjustSize()
        self.__textLabel.updateGeometry()
        self.adjustSize()

    def text(self):
        """This property holds the message box text to be displayed.
        The default value of this property is an empty string."""
        return self.__textLabel.text()

    def setIcon(self, icon):
        """Set icon for message box.

        Args:
            icon (QIcon): Message box icon.
        """
        if icon:
            icon = QApplication.instance().style().standardIcon(icon)
            self.__iconLabel.setPixmap(icon.pixmap(QSize(128, 128)))
        else:
            self.__iconLabel.setPixmap(QIcon().pixmap(QSize(128, 128)))

    def icon(self):
        """This property holds the message box's icon"""
        return QIcon(self.__iconLabel.pixmap())


class FramelessWarningMessageBox(FramelessMessageBox):
    def __init__(self, parent=None):
        super(FramelessWarningMessageBox, self).__init__(icon=QStyle.SP_MessageBoxWarning, parent=parent)


class FramelessInformationMessageBox(FramelessMessageBox):
    def __init__(self, parent=None):
        super(FramelessInformationMessageBox, self).__init__(icon=QStyle.SP_MessageBoxInformation, parent=parent)


class FramelessCriticalMessageBox(FramelessMessageBox):
    def __init__(self, parent=None):
        super(FramelessCriticalMessageBox, self).__init__(icon=QStyle.SP_MessageBoxCritical, parent=parent)


class FramelessQuestionMessageBox(FramelessMessageBox):
    def __init__(self, parent=None):
        super(FramelessQuestionMessageBox, self).__init__(icon=QStyle.SP_MessageBoxQuestion, parent=parent)
